/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
/*---Shell Includes---------------------------------------------------*/
#include "../sdk/include/plsdefs.h"
#include "../sdk/include/plsapi.h"
#include "config.h"
/*--------------------------------------------------------------------*/


/*---Global Variables-------------------------------------------------*/
CRITICAL_SECTION g_csDataAccess;
HINSTANCE g_hMainInst; /* Unused */

cfgSettings g_cfgs;
memBuffers g_mbf;
/*--------------------------------------------------------------------*/



/*--------SH_CFG_IsBufferOpen-------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_IsBufferOpen(
	VOID
)
{
	BOOL bIsOpen;

	EnterCriticalSection(&g_csDataAccess);

	bIsOpen = NULL != g_mbf.pszBufCFG;

	LeaveCriticalSection(&g_csDataAccess);

	return( bIsOpen );
}


/*--------SH_CFG_FreeBuffer---------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID WINAPI SH_CFG_FreeBuffer(
	VOID
)
{
	EnterCriticalSection(&g_csDataAccess);

	if( NULL != g_mbf.pConfigPath )
	{
		free(g_mbf.pConfigPath);
		g_mbf.pConfigPath = NULL;
	}

	if( NULL != g_mbf.pConfigSrc )
	{
		free(g_mbf.pConfigSrc);
		g_mbf.pConfigSrc = NULL;
	}

	freeBuffer( &g_mbf.pszModules, &g_mbf.nModules );
	freeBuffer( &g_mbf.pszModFolders, &g_mbf.nModFolders );
	freeBuffer( &g_mbf.pszFiles, &g_mbf.nFiles );
	freeBuffer( &g_mbf.pszFileFolders, &g_mbf.nFileFolders );
	freeBuffer( &g_mbf.pszBufCFG, &g_mbf.nBufCFG );
	freeBuffer( &g_mbf.pszBufVAR, &g_mbf.nBufVAR );

	memset(&g_mbf, 0, sizeof(memBuffers));
	memset(&g_cfgs, 0, sizeof(cfgSettings));

	LeaveCriticalSection(&g_csDataAccess);

	return;
}


/*--------SH_CFG_FillBuffer---------------------------------------------
 Purpose:
 - Load config buffers.  If specified or default "purels.cfg" can not
 be loaded, then try to load a "step.rc" in Legacy mode.
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_FillBuffer(
	LPCTSTR pCfgPath,
	LPCTSTR pCfgSrc
)
{
	if( SH_CFG_IsBufferOpen() )
	{
		return( TRUE );
	}

	EnterCriticalSection(&g_csDataAccess);

	if( setupConfigPath(pCfgPath, pCfgSrc) && parseDefaultConfig() )
	{
		if( !parseDefinedConfig() )
		{
			parseLegacyConfig();
		}
	}

	if( NULL == g_mbf.pszBufCFG )
	{
		SH_CFG_FreeBuffer();
	}
	else
	{
		if( !g_cfgs.bDisableExternalLoadModules )
		{
			if(
				!buildFileList(
					&g_mbf.pszModules,
					&g_mbf.nModules,
					_T("LoadModule"),
					(LPCTSTR*)g_mbf.pszModFolders,
					g_mbf.nModFolders,
					TRUE
				)
			)
			{
				SH_CFG_FreeBuffer();
			}
		}
	}

	LeaveCriticalSection(&g_csDataAccess);

	return( SH_CFG_IsBufferOpen() );
}


/*--------SH_CFG_RefillBuffer-------------------------------------------
 Purpose:
 - Completely reload the current configuration
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_RefillBuffer(
	VOID
)
{
	BOOL bSuccess = FALSE;

	if(SH_CFG_IsBufferOpen())
	{
		LPTSTR pszCfgPath, pszCfgSrc;

		EnterCriticalSection(&g_csDataAccess);

		pszCfgPath = (LPTSTR)malloc(
			(_tcslen(g_mbf.pConfigPath) + 1) * sizeof(_TCHAR)
		);

		if( NULL != pszCfgPath )
		{
			_tcscpy(pszCfgPath, g_mbf.pConfigPath);
		}

		pszCfgSrc = (LPTSTR)malloc(
			(_tcslen(g_mbf.pConfigSrc) + 1) * sizeof(_TCHAR)
		);

		if( NULL != pszCfgSrc )
		{
			_tcscpy(pszCfgSrc, g_mbf.pConfigSrc);
		}

		LeaveCriticalSection(&g_csDataAccess);

		SH_CFG_FreeBuffer();

		bSuccess = SH_CFG_FillBuffer(pszCfgPath, pszCfgSrc);

		if(NULL != pszCfgPath)
		{
			free(pszCfgPath);
		}

		if(NULL != pszCfgSrc)
		{
			free(pszCfgSrc);
		}
	}

	return(bSuccess);
}


/*********************************************************/
/** Internal Information / Status Retrieval functions ****/
/*********************************************************/

/*--------SH_CFG_GetFileFolder------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetFileFolder(
	UINT* nStartPos,
	LPTSTR destBuf
)
{
	BOOL bSuccess;

	EnterCriticalSection(&g_csDataAccess);

	bSuccess = (
		NULL != g_mbf.pszFileFolders
		&&
		enumList(
			nStartPos,
			destBuf,
			(LPCTSTR*)g_mbf.pszFileFolders,
			g_mbf.nFileFolders
		)
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bSuccess );
}


/*--------SH_CFG_GetModFolder-------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetModFolder(
	UINT* nStartPos,
	LPTSTR destBuf
)
{
	BOOL bSuccess;

	EnterCriticalSection(&g_csDataAccess);

	bSuccess =(
		NULL != g_mbf.pszModFolders
		&&
		enumList(
			nStartPos,
			destBuf,
			(LPCTSTR*)g_mbf.pszModFolders,
			g_mbf.nModFolders
		)
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bSuccess );
}


/*--------SH_CFG_GetConfigFile------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetConfigFile(
	UINT* nStartPos,
	LPTSTR destBuf
)
{
	BOOL bSuccess;

	EnterCriticalSection(&g_csDataAccess);

	bSuccess =(
		NULL != g_mbf.pszFiles
		&&
		enumList(
			nStartPos,
			destBuf,
			(LPCTSTR*)g_mbf.pszFiles,
			g_mbf.nFiles
		)
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bSuccess );
}


/*--------SH_CFG_GetLoadModule------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetLoadModule(
	UINT* nStartPos,
	LPTSTR destBuf
)
{
	BOOL bSuccess;

	EnterCriticalSection(&g_csDataAccess);

	bSuccess =(
		NULL != g_mbf.pszModules
		&&
		enumList(
			nStartPos,
			destBuf,
			(LPCTSTR*)g_mbf.pszModules,
			g_mbf.nModules
		)
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bSuccess );
}



/*********************************************************/
/******** Data Retrieval functions ***********************/
/*********************************************************/


/*--------SH_CFG_FindLine-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_FindLine(
	UINT*   nStartPos,
	LPCTSTR srcStr
)
{
	BOOL bSuccess;

	EnterCriticalSection(&g_csDataAccess);

	bSuccess =(
		NULL != nStartPos && NULL != srcStr
		?
		NULL != searchBuf(nStartPos, srcStr) : FALSE
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bSuccess );
}


/*--------SH_CFG_GetLine------------------------------------------------
 Purpose:
 -

 NOTE:
 - Ignores g_cfgs.bDisableEVars and uses bParse specified by caller
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetLine(
	UINT*   nStartPos,
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	size_t  nMaxLen,
	BOOL    bNoParse
)
{
	BOOL bResult = FALSE;

	if(
		(NULL != destBuf || (NULL != srcStr && 0 != *srcStr))
		&&
		nMaxLen > 0
	)
	{
		LPTSTR memPtr;

		EnterCriticalSection(&g_csDataAccess);

		memPtr = searchBuf(nStartPos, srcStr);

		if( NULL != memPtr )
		{
//			copyBuffer(
//				destBuf,
//				memPtr,
//				nMaxLen,
//				bNoParse,
//				FALSE
//			);
			if( !bNoParse )
			{
				SH_CFG_ExpandBuffer(destBuf, memPtr, nMaxLen);
			}
			else
			{
				_tcsncpy(destBuf, memPtr, nMaxLen);
				destBuf[nMaxLen-1] = 0;
			}

			bResult = TRUE;
		}

		LeaveCriticalSection(&g_csDataAccess);
	}

	return( bResult );
}


/*--------SH_CFG_GetParam-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetParam(
	UINT*   nStartPos,
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	size_t  nMaxLen,
	BOOL    bNoParse
)
{
	BOOL bResult = FALSE;

	if(
		NULL != destBuf
		&&
		NULL != srcStr && 0 != *srcStr
		&&
		nMaxLen > 0
	)
	{
		LPTSTR memPtr;

		EnterCriticalSection(&g_csDataAccess);

		memPtr = searchBuf(nStartPos, srcStr);

		if( NULL != memPtr )
		{
			copyBuffer(
				destBuf,
				(memPtr += _tcslen(srcStr)),
				nMaxLen,
				g_cfgs.bDisableEVars || bNoParse,
				FALSE
			);

			bResult = TRUE;
		}

		LeaveCriticalSection(&g_csDataAccess);
	}

	return( bResult );
}


/*--------SH_CFG_GetParam-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetVar(
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	size_t  nMaxLen
)
{
	BOOL bResult = FALSE;

	if(
		NULL != destBuf
		&&
		NULL != srcStr && 0 != *srcStr
		&&
		nMaxLen > 0
	)
	{
		LPTSTR memPtr;

		EnterCriticalSection(&g_csDataAccess);

		memPtr = searchVar(srcStr);

		if( NULL != memPtr )
		{
			copyBuffer(
				destBuf,
				(memPtr += _tcslen(srcStr)),
				nMaxLen,
				FALSE,
				FALSE
			);

			bResult = TRUE;
		}

		LeaveCriticalSection(&g_csDataAccess);
	}

	return( bResult );
}

VOID WINAPI SH_CFG_RegisterVar(
	LPCTSTR srcStr
)
{
	if( NULL != srcStr && 0 != *srcStr )
	{
		_TCHAR retBuf[PLS_MAX_SIZE] = {0};

		PL_SimpleToken(retBuf, NULL, srcStr);

		EnterCriticalSection(&g_csDataAccess);

		delVar(retBuf);

		g_mbf.pszBufVAR = allocateBuffer(g_mbf.pszBufVAR, &g_mbf.nBufVAR, srcStr);

		LeaveCriticalSection(&g_csDataAccess);
	}

	return;
}

VOID WINAPI SH_CFG_UnregisterVar(
	LPCTSTR srcStr
)
{
	EnterCriticalSection(&g_csDataAccess);

	delVar(srcStr);

	LeaveCriticalSection(&g_csDataAccess);

	return;
}

/*--------SH_CFG_ReadParam----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_ReadParam(
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	LPCTSTR searchStr,
	size_t  nMaxLen,
	BOOL    bNoParse
)
{
	BOOL bResult = FALSE;

	if(
		NULL != destBuf
		&&
		NULL != srcStr && 0 != *srcStr
		&&
		NULL != searchStr && 0 != *searchStr
		&&
		_tcslen(srcStr) >= _tcslen(searchStr)
		&&
		nMaxLen > 0
	)
	{
		copyBuffer(
			destBuf,
			(srcStr += _tcslen(searchStr)),
			nMaxLen,
			g_cfgs.bDisableEVars || bNoParse,
			FALSE
		);

		bResult = TRUE;
	}

	return( bResult );
}


/*--------SH_CFG_GetString----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetString(
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	LPCTSTR defVal,
	size_t  nMaxLen
)
{
	BOOL bResult = FALSE;

	if(
		NULL != destBuf
		&&
		NULL != srcStr && 0 != *srcStr
		&&
		nMaxLen > 0
	)
	{
		LPTSTR memPtr;

		EnterCriticalSection(&g_csDataAccess);

		memPtr = searchBuf(NULL, srcStr);

		if( NULL != memPtr )
		{
			if( 0 != *(memPtr += _tcslen(srcStr)) )
			{
				copyBuffer(
					destBuf,
					memPtr,
					nMaxLen,
					g_cfgs.bDisableEVars,
					TRUE
				);

				bResult = TRUE;
			}
		}

		LeaveCriticalSection(&g_csDataAccess);

		if( FALSE == bResult && NULL != defVal )
		{
			/*----------------------------------------------------------
			 Set last param to FALSE - we don't want to parse it
			----------------------------------------------------------*/
			copyBuffer(
				destBuf,
				defVal,
				nMaxLen,
				g_cfgs.bDisableEVars,
				FALSE
			);
		}
	}

	return( bResult );
}


/*--------SH_CFG_ReadString---------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_ReadString(
	LPTSTR  destBuf,
	LPCTSTR srcStr,
	LPCTSTR searchStr,
	LPCTSTR defVal,
	size_t  nMaxLen
)
{
	BOOL bResult = FALSE;

	if(
		NULL != destBuf
		&&
		NULL != srcStr && 0 != *srcStr
		&&
		NULL != searchStr && 0 != *searchStr
		&&
		_tcslen(srcStr) >= _tcslen(searchStr)
		&&
		nMaxLen > 0
	)
	{
		LPCTSTR memPtr;

		memPtr = srcStr + _tcslen(searchStr);

		if( 0 != *memPtr )
		{
			copyBuffer(
				destBuf,
				memPtr,
				nMaxLen,
				g_cfgs.bDisableEVars,
				TRUE
			);

			bResult = TRUE;
		}

		if( FALSE == bResult && NULL != defVal )
		{
			/*----------------------------------------------------------
			 Set last param to FALSE - we don't want to parse it
			----------------------------------------------------------*/
			copyBuffer(
				destBuf,
				defVal,
				nMaxLen,
				g_cfgs.bDisableEVars,
				FALSE
			);
		}
	}

	return( bResult );
}


/*--------SH_CFG_GetBool------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_GetBool(
	LPCTSTR srcStr
)
{
	BOOL bFound;

	EnterCriticalSection(&g_csDataAccess);

	bFound =(
		NULL != srcStr && 0 != *srcStr
		&&
		NULL != searchBuf(NULL, srcStr)
	);

	LeaveCriticalSection(&g_csDataAccess);

	return( bFound );
}


/*--------SH_CFG_ReadBool-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI SH_CFG_ReadBool(
	LPCTSTR srcStr,
	LPCTSTR searchStr
)
{
	BOOL bFound;

	bFound =(
		NULL != srcStr && 0 != *srcStr
		&&
		NULL != searchStr && 0 != *searchStr
	);

	return( bFound );
}


/*--------SH_CFG_GetInt-------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
INT WINAPI SH_CFG_GetInt(
	LPCTSTR srcStr,
	INT     defVal
)
{
	_TCHAR tmpBuf[12];

	if( SH_CFG_GetString(tmpBuf, srcStr, NULL, 12) )
	{
		if(
			0 != *tmpBuf
			&&
			(
				_istdigit(*tmpBuf)
				||
				(_T('-') == *tmpBuf && _istdigit(*(tmpBuf+1)))
			)
		)
		{
			tmpBuf[_T('-') == *tmpBuf ? 11:10] = 0;
			defVal = _ttoi(tmpBuf);
		}
	}

	return( defVal );
}


/*--------SH_CFG_ReadInt------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
INT WINAPI SH_CFG_ReadInt(
	LPCTSTR srcStr,
	LPCTSTR searchStr,
	INT     defVal
)
{
	_TCHAR tmpBuf[12];

	if( SH_CFG_ReadString(tmpBuf, srcStr, searchStr, NULL, 12) )
	{
		if(
			0 != *tmpBuf
			&&
			(
				_istdigit(*tmpBuf)
				||
				(_T('-') == *tmpBuf && _istdigit(*(tmpBuf+1)))
			)
		)
		{
			tmpBuf[_T('-') == *tmpBuf ? 11:10] = 0;
			defVal = _ttoi(tmpBuf);
		}
	}

	return( defVal );
}


/*--------SH_CFG_GetColor-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
COLORREF WINAPI SH_CFG_GetColor(
	LPCTSTR  srcStr,
	COLORREF defVal
)
{
	_TCHAR tmpBuf[20];

	if( SH_CFG_GetString(tmpBuf, srcStr, NULL, 20) )
	{
		defVal = (COLORREF)_tcstol(tmpBuf, NULL, 16);

		if( !g_cfgs.bColorBGR )
		{
			defVal = RGB(
				GetBValue(defVal),
				GetGValue(defVal),
				GetRValue(defVal)
			);
		}
	}

	return( defVal );
}


/*--------SH_CFG_ReadColor----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
COLORREF WINAPI SH_CFG_ReadColor(
	LPCTSTR  srcStr,
	LPCTSTR  searchStr,
	COLORREF defVal
)
{
	_TCHAR tmpBuf[20];

	if( SH_CFG_ReadString(tmpBuf, srcStr, searchStr, NULL, 20) )
	{
		defVal = (COLORREF)_tcstol(tmpBuf, NULL, 16);

		if( !g_cfgs.bColorBGR )
		{
			defVal = RGB(
				GetBValue(defVal),
				GetGValue(defVal),
				GetRValue(defVal)
			);
		}
	}

	return( defVal );
}


/*********************************************************/
/******** Data Manipulation functions ********************/
/*********************************************************/


/*--------SH_CFG_ExpandBuffer-------------------------------------------
 Purpose:
 -

 NOTE:
 - Should look into thinning this down if possible
----------------------------------------------------------------------*/
LPTSTR WINAPI SH_CFG_ExpandBuffer(
	LPTSTR  destBuf,
	LPCTSTR srcBuf,
	size_t  nMaxLen
)
{
	if( destBuf != NULL && srcBuf != NULL && nMaxLen > 1 )
	{
		LPTSTR ptr1 = NULL;
		LPTSTR ptr2 = NULL;

		if(
			(ptr1 = _tcschr(srcBuf, _T('$'))) != NULL
			&&
			_tcsrchr(srcBuf, _T('$')) != ptr1
		)
		{
			_TCHAR temp[PLS_MAX_SIZE] = {0};
			_TCHAR buffer[PLS_MAX_SIZE] = {0};
			LPTSTR pEnvVar = NULL;

			*(_tcscpy(buffer, srcBuf) + (ptr1++ - srcBuf)) = 0;
			ptr2 = _tcschr(ptr1, _T('$'));
			_tcsncpy(temp, ptr1, ptr2++ - ptr1);
			*(temp + (ptr2-ptr1-1)) = 0;

			if( SH_CFG_GetVar(temp, temp, PLS_MAX_SIZE) )
			{
				_tcscat(buffer, temp);
			}
			else if( SH_CFG_GetParam(NULL, temp, temp, PLS_MAX_SIZE, FALSE) )
			{
				_tcscat(buffer, temp);
			}
			else if( (pEnvVar = _tgetenv(temp)) != NULL )
			{
				_tcscat(buffer, pEnvVar);
			}
			else
			{
				*(buffer + (ptr1 - 1 - srcBuf)) = _T('$');
				*(buffer + (ptr2 - srcBuf)) = 0;
			}

			if( *ptr2 != 0 )
			{
				if( _tcschr(ptr2, _T('$')) != NULL )
				{
					_tcsncat(
						buffer,
						SH_CFG_ExpandBuffer(temp, ptr2, PLS_MAX_SIZE),
						PLS_MAX_SIZE
					);
				}
				else
				{
					_tcscat(buffer, ptr2);
				}
			}

			PL_StripWS(buffer);
			_tcsncpy(destBuf, buffer, nMaxLen);
			destBuf[nMaxLen-1] = 0;
		}
		else
		{
			_tcsncpy(destBuf, srcBuf, nMaxLen);
			destBuf[nMaxLen-1] = 0;
			PL_StripWS(destBuf);
		}
	}

	return( destBuf );
}

/*--------DllMain-------------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL WINAPI DllMain(
	HINSTANCE hinstDLL,
	DWORD fdwReason,
	LPVOID lpvReserved
)
{
	BOOL retVal = TRUE;

	switch( fdwReason )
	{
	case DLL_PROCESS_ATTACH:
		if( DisableThreadLibraryCalls(hinstDLL) )
		{
			g_hMainInst = hinstDLL;
			InitializeCriticalSection(&g_csDataAccess);
		}
		else
		{
			retVal = FALSE;
		}
		break;

	case DLL_PROCESS_DETACH:
		DeleteCriticalSection(&g_csDataAccess);
		g_hMainInst = NULL;
		break;
	}

	return( retVal );
	UNREFERENCED_PARAMETER(lpvReserved);
}

/*--------End Of File-------------------------------------------------*/
