/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
/*---Shell Includes---------------------------------------------------*/
#include "../sdk/include/plsdefs.h"
#include "../sdk/include/plsapi.h"
#include "config.h"
/*--------------------------------------------------------------------*/


/*---Global Variables-------------------------------------------------*/
extern cfgSettings g_cfgs;
extern memBuffers g_mbf;

static LPCTSTR CONST PURELS_DEFAULT_CONFIG_FILE = _T("purels.cfg");
/*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*/
BOOL setupConfigPath(
	LPCTSTR pCfgPath,
	LPCTSTR pCfgSrc
)
{
	if( NULL != pCfgPath && 0 != *pCfgPath )
	{
		g_mbf.pConfigPath = (LPTSTR)malloc(
			(_tcslen(pCfgPath) + 1) * sizeof( _TCHAR )
		);

		if( NULL != g_mbf.pConfigPath )
		{
			_tcscpy(g_mbf.pConfigPath, pCfgPath);
		}
	}

	if( NULL == g_mbf.pConfigPath )
	{
		return( FALSE );
	}

	if( NULL == pCfgSrc || 0 == *pCfgSrc )
	{
		pCfgSrc = PURELS_DEFAULT_CONFIG_FILE;
	}

	if( NULL == _tcschr(pCfgSrc, _T('\\')) )
	{
		_TCHAR buffer[PLS_MAX_SIZE];

		_stprintf(
			buffer,
			_T("%s\\%s"),
			g_mbf.pConfigPath,
			pCfgSrc
		);

		g_mbf.pConfigSrc = (LPTSTR)malloc(
			(_tcslen(buffer) + 1) * sizeof( _TCHAR )
		);

		if( NULL != g_mbf.pConfigSrc )
		{
			_tcscpy(g_mbf.pConfigSrc, buffer);
		}
	}
	else
	{
		g_mbf.pConfigSrc = (LPTSTR)malloc(
			(_tcslen(pCfgSrc) + 1) * sizeof( _TCHAR )
		);

		if( NULL != g_mbf.pConfigSrc )
		{
			_tcscpy(g_mbf.pConfigSrc, pCfgSrc);
		}
	}

	return( NULL != g_mbf.pConfigSrc );
}


/*--------------------------------------------------------------------*/
BOOL loadConfigBuffer( LPCTSTR pFile, BOOL bComments )
{
	FILE* fCFG;
	_TCHAR buffer[PLS_MAX_SIZE];

	fCFG = PL_fOpen(pFile);

	if( NULL != fCFG )
	{
		while( PL_fGetNextLine(fCFG, 0, buffer, PLS_MAX_SIZE) )
		{
			g_mbf.pszBufCFG = allocateBuffer(
				g_mbf.pszBufCFG,
				&g_mbf.nBufCFG,
				(
					bComments
					?
					removeComment(buffer):buffer
				)
			);

			if( NULL == g_mbf.pszBufCFG )
			{
				break;
			}
		}

		PL_fClose(fCFG);
	}

	return( NULL != g_mbf.pszBufCFG );
}


/*--------------------------------------------------------------------*/
BOOL parseDefaultConfig( VOID )
{
	if( loadConfigBuffer(g_mbf.pConfigSrc, TRUE) )
	{
		g_cfgs.bDisableEVars = SH_CFG_GetBool(
			_T("DisableEVars")
		);
		g_cfgs.bEnableEndLineComments = SH_CFG_GetBool(
			_T("EnableEndLineComments")
		);
		g_cfgs.bDisableExternalLoadModules = SH_CFG_GetBool(
			_T("DisableExternalLoadModules")
		);
		g_cfgs.bColorBGR = SH_CFG_GetBool(
			_T("ColorBGR")
		);

		if(
			!buildVarList(
				 &g_mbf.pszBufVAR
				,&g_mbf.nBufVAR
				,_T("RegisterVar")
			)
			||
			!buildFolderList(
				&g_mbf.pszFileFolders,
				&g_mbf.nFileFolders,
				_T("ConfigFileFolder")
			)
			||
			!buildFolderList(
				&g_mbf.pszModFolders,
				&g_mbf.nModFolders,
				_T("LoadModuleFolder")
			)
			||
			!buildFileList(
				&g_mbf.pszFiles,
				&g_mbf.nFiles,
				_T("ConfigFile"),
				(LPCTSTR*)g_mbf.pszFileFolders,
				g_mbf.nFileFolders,
				TRUE
			)
			||
			!buildFileList(
				&g_mbf.pszModules,
				&g_mbf.nModules,
				_T("LoadModule"),
				(LPCTSTR*)g_mbf.pszModFolders,
				g_mbf.nModFolders,
				TRUE
			)
		)
		{
			return( FALSE );
		}


		/*--------------------------------------------------------------
		 Free temporary config file buffer.
		--------------------------------------------------------------*/
		freeBuffer( &g_mbf.pszBufCFG, &g_mbf.nBufCFG );
	}

	return( TRUE );
}


/*--------------------------------------------------------------------*/
BOOL parseDefinedConfig( VOID )
{
	if( NULL != g_mbf.pszFiles )
	{
		UINT i;

		for( i = 0; i < g_mbf.nFiles; i++ )
		{
			loadConfigBuffer(
				g_mbf.pszFiles[i],
				g_cfgs.bEnableEndLineComments
			);
		}	
	}

	return( NULL != g_mbf.pszBufCFG );
}


/*--------------------------------------------------------------------*/
BOOL parseLegacyConfig( VOID )
{
	_TCHAR buffer[PLS_MAX_SIZE];

	/*------------------------------------------------------------------
	 Look for a "step.rc" file for legacy setups.
	------------------------------------------------------------------*/
	_stprintf(
		buffer,
		_T("%s\\%s"),
		g_mbf.pConfigPath,
		_T("step.rc")
	);

	if( loadConfigBuffer(buffer, g_cfgs.bEnableEndLineComments) )
	{
		if( !g_cfgs.bColorBGR )
		{
			g_cfgs.bColorBGR = SH_CFG_GetBool( _T("LSColorBGR") );
		}
	}

	return( NULL != g_mbf.pszBufCFG );
}

/*--------End Of File-------------------------------------------------*/
