/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
/*---Shell Includes---------------------------------------------------*/
#include "../sdk/include/plsdefs.h"
#include "../sdk/include/plsapi.h"
#include "config.h"
/*--------------------------------------------------------------------*/


/*---Global Variables-------------------------------------------------*/
extern cfgSettings g_cfgs;
extern memBuffers g_mbf;
/*--------------------------------------------------------------------*/



/*--------removeComment-------------------------------------------------
 Purpose:
 - Removes "end line" comments (anything following a semicolon)

   SomeCommand ;this IS a comment
   AnotherCommand ";this is NOT a comment"
   NewCommand [;this is NOT a comment]
   AgainCommand [something "[but]" ;this is NOT a comment]
   LastCommand [something [but] ;this IS a comment]
   ^
   ^--- LastCommand has a comment - Nested brackets are not supported.
----------------------------------------------------------------------*/
LPTSTR removeComment(
	LPTSTR buffer
)
{
	LPTSTR ptr = buffer;
	UINT nBrackets = 0;
	UINT nQuotes = 0;

	while( 0 != *ptr )
	{
		if( _T(';') == *ptr )
		{
			if( !nBrackets && !nQuotes )
			{
				while( buffer != ptr && _istspace(*(ptr - 1)) )
				{
					--ptr;
				}

				*ptr = 0;
				break;
			}
		}
		else if( _T('\"') == *ptr )
		{
			nQuotes = nQuotes ? 0:1;
		}
		else if(0 == nQuotes)
		{
			if( _T('[') == *ptr )
			{
				nBrackets = 1;
				//nBrackets++;
			}
			else if( _T(']') == *ptr )
			{
				nBrackets = 0;
				//if(0 != nBrackets) nBrackets--;
			}
		}

		ptr++;
	}

	return( buffer );
}

/*--------copyBuffer----------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
VOID copyBuffer(
	LPTSTR destBuf,
	LPCTSTR srcBuf,
	UINT nMaxLen,
	BOOL bNoParse,
	BOOL bToken
)
{
	if( *srcBuf != 0 )
	{
		_TCHAR retBuf[PLS_MAX_SIZE] = {0};

		if( !bNoParse )
		{
			SH_CFG_ExpandBuffer(retBuf, srcBuf, PLS_MAX_SIZE);
		}
		else
		{
			_tcsncpy(retBuf, srcBuf, PLS_MAX_SIZE);
			retBuf[PLS_MAX_SIZE-1] = 0;
			PL_StripWS(retBuf);
		}

		if(
			bToken
			?
			PL_SimpleToken(retBuf, NULL, retBuf) : 0 != *retBuf
		)
		{
			_tcsncpy(destBuf, retBuf, nMaxLen);
			destBuf[nMaxLen-1] = 0;
		}
	}
	else
	{
		*destBuf = 0;
	}

	return;
}


/*--------enumList------------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL enumList(
	UINT* nStartPos,
	LPTSTR destBuf,
	LPCTSTR* pszBuf,
	UINT nBuf
)
{
	/*------------------------------------------------------------------
	 Return size of the buffer, if destBuf is NULL
	------------------------------------------------------------------*/
	if( nStartPos != NULL && destBuf == NULL )
	{
		*nStartPos = nBuf;
	}
	else if( pszBuf != NULL && nStartPos != NULL && *nStartPos < nBuf )
	{
		_tcscpy(destBuf, pszBuf[(*nStartPos)++]);
		return( TRUE );
	}

	return( FALSE );
}


/*--------isValidFile---------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL isValidFile(
	LPCTSTR pszFile
)
{
	DWORD dwAtt = GetFileAttributes(pszFile);

	return(
		0xFFFFFFFF != dwAtt
		&&
		FILE_ATTRIBUTE_DIRECTORY != (dwAtt & FILE_ATTRIBUTE_DIRECTORY)
	);
}


/*--------isValidFolder-------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL isValidFolder(
	LPCTSTR pszFile
)
{
	DWORD dwAtt = GetFileAttributes(pszFile);

	return(
		0xFFFFFFFF != dwAtt
		&&
		FILE_ATTRIBUTE_DIRECTORY == (dwAtt & FILE_ATTRIBUTE_DIRECTORY)
	);
}


/*--------buildFolderList-----------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL buildFolderList(
	LPTSTR** pszBuf,
	UINT* nBuf,
	LPCTSTR src
)
{
	if( pszBuf != NULL && nBuf != NULL )
	{
		_TCHAR buffer[PLS_MAX_SIZE] = {0};
		UINT i=0;

		while( SH_CFG_GetParam(&i, buffer, src, MAX_PATH, FALSE) )
		{
			PL_SimpleToken(buffer, NULL, buffer);

			if( *(buffer+_tcslen(buffer)-1) != _T('\\') )
			{
				_tcscat(buffer, _T("\\"));
			}

			if( isValidFolder(buffer) )
			{
				*pszBuf = allocateBuffer(*pszBuf, nBuf, buffer);

				if( NULL == *pszBuf )
				{
					return FALSE;
				}
			}
			else
			{
				MessageBoxPrintf(
					MB_OK|MB_ICONERROR|MB_DEFBUTTON1|MB_SYSTEMMODAL,
					_T("PureLS - Directory not found!"),
					_T("%s:\n\n\"%s\"\n\ndoes not exist!"),
					src,
					buffer
				);
			}

			*buffer = 0;
		}
	}

	return( TRUE );
}


/*--------buildFileList-------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL buildFileList(
	LPTSTR** pszBuf,
	UINT* nBuf,
	LPCTSTR src,
	LPCTSTR* pszFolderList,
	UINT nFolderList,
	BOOL bMustExist
)
{
	if( pszBuf != NULL && nBuf != NULL )
	{
		_TCHAR buffer[PLS_MAX_SIZE] = {0};
		_TCHAR fileBuf[MAX_PATH] = {0};
		UINT i = 0;
		UINT n;
		BOOL bFound;

		while( SH_CFG_GetParam(&i, buffer, src, MAX_PATH, FALSE) )
		{
			bFound = FALSE;
			PL_SimpleToken(fileBuf, NULL, buffer);

			if( _tcschr(fileBuf, _T('\\')) == NULL )
			{
				if( pszFolderList != NULL && nFolderList > 0 )
				{
					for( n = 0; !bFound && n<nFolderList; n++ )
					{
						_stprintf(
							buffer,
							_T("%s%s"),
							pszFolderList[n],
							fileBuf
						);

						if( isValidFile(buffer) )
						{
							_tcscpy(fileBuf, buffer);
							bFound = TRUE;
						}
					}
				}

				if( !bFound )
				{
					_stprintf(
						buffer,
						_T("%s\\%s"),
						g_mbf.pConfigPath,
						fileBuf
					);

					if( isValidFile(buffer) )
					{
						_tcscpy(fileBuf, buffer);
						bFound = TRUE;
					}
				}
			}
			else if( isValidFile(fileBuf) )
			{
				bFound = TRUE;
			}

			if( !bMustExist || bFound )
			{
				*pszBuf = allocateBuffer(*pszBuf, nBuf, fileBuf);

				if( NULL == *pszBuf )
				{
					return( FALSE );
				}
			}
			else
			{
				MessageBoxPrintf(
					MB_OK|MB_ICONERROR|MB_DEFBUTTON1|MB_SYSTEMMODAL,
					_T("PureLS - File not found!"),
					_T("%s:\n\n\"%s\"\n\ndoes not exist!"),
					src,
					fileBuf
				);
			}
		}
	}

	return( TRUE );
}


/*--------buildVarList--------------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL buildVarList(
	LPTSTR** pszBuf,
	UINT* nBuf,
	LPCTSTR src
)
{
	if( pszBuf != NULL && nBuf != NULL )
	{
		_TCHAR buffer[PLS_MAX_SIZE] = {0};
		UINT i=0;

		while( SH_CFG_GetParam(&i, buffer, src, PLS_MAX_SIZE, TRUE) )
		{
			*pszBuf = allocateBuffer(*pszBuf, nBuf, buffer);

			if( NULL == *pszBuf )
			{
				return( FALSE );
			}

			*buffer = 0;
		}
	}

	return( TRUE );
}

/*--------allocateBuffer------------------------------------------------
 Purpose:
 - Allocate buffer space
----------------------------------------------------------------------*/
LPTSTR* allocateBuffer(
	LPTSTR* pszBuf,
	UINT* nBuf,
	LPCTSTR src
)
{
	LPTSTR tmpPtr = NULL;
	LPTSTR *tmpBuf = NULL;

	tmpPtr = (LPTSTR)malloc((_tcslen(src) + 1) * sizeof( _TCHAR ));

	if( NULL == tmpPtr )
	{
		if( NULL != pszBuf )
		{
			/*----------------------------------------------------------
			 Free anything we've allocated up to this point
			----------------------------------------------------------*/
			freeBuffer( &pszBuf, nBuf );
		}

		*nBuf = 0;
	}
	else
	{
		_tcscpy(tmpPtr, src);

		if( NULL == pszBuf )
		{
			*nBuf = 0;
			pszBuf = (LPTSTR*)malloc(sizeof( LPTSTR ));
		}
		else
		{
			/*----------------------------------------------------------
			 Store current pointer, since if realloc fails, tmpBuf will
			 still point to a valid memory location of the buffer.
			----------------------------------------------------------*/
			tmpBuf = pszBuf;
			pszBuf = (LPTSTR*)realloc(
				pszBuf,
				(*nBuf + 1) * sizeof( LPTSTR )
			);
		}

		/*--------------------------------------------------------------
		 Lets clean up so we don't leave memory leaks around.
		--------------------------------------------------------------*/
		if( NULL == pszBuf )
		{
			if( NULL != tmpPtr )
			{
				free(tmpPtr);
			}

			if( NULL != tmpBuf )
			{
				freeBuffer( &tmpBuf, nBuf );
			}

			*nBuf = 0;
		}
		else
		{
			pszBuf[(*nBuf)++] = tmpPtr;
		}
	}

	return( pszBuf );
}


/*--------freeBuffer----------------------------------------------------
 Purpose:
 - Free buffer space
----------------------------------------------------------------------*/
VOID freeBuffer(
	LPTSTR** pszBuf,
	UINT* nBuf
)
{
	UINT i;

	if( NULL != *pszBuf )
	{
		for( i = 0; i < *nBuf; i++ )
		{
			if( NULL != (*pszBuf)[i] )
			{
				free((*pszBuf)[i]);
				(*pszBuf)[i] = NULL;
			}
		}

		free(*pszBuf);
		*pszBuf = NULL;
	}

	*nBuf = 0;
	
	return;
}

/*--------searchBuf-----------------------------------------------------
 Purpose:
 - Finds the given string, starting at the specified position

 NOTES:
 - If nStartPos is NULL, search starts at beginning of buffer.
 - If string is NULL, the next item is returned.
----------------------------------------------------------------------*/
LPTSTR searchBuf(
	UINT* const nStartPos,
	LPCTSTR const string
)
{
	UINT i;

	if( NULL != nStartPos )
	{
		if( *nStartPos >= g_mbf.nBufCFG )
		{
			return( NULL );
		}
		
		i = *nStartPos;
	}
	else
	{
		i = 0;
	}

	if( NULL != string && 0 != *string )
	{
		size_t len = _tcslen(string);

		while(
			i < g_mbf.nBufCFG
			&&
			!(
				0 == _tcsnicmp(g_mbf.pszBufCFG[i], string, len)
				&&
				(
					_istspace(*(g_mbf.pszBufCFG[i] + len))
					||
					0 == *(g_mbf.pszBufCFG[i] + len)
				)
			)
		) ++i;
	}
	else if( NULL == nStartPos )
	{
		i = g_mbf.nBufCFG;
	}

	if( NULL != nStartPos && i < g_mbf.nBufCFG )
	{
		*nStartPos = i+1;
	}

	return( i == g_mbf.nBufCFG ? NULL:g_mbf.pszBufCFG[i] );
}

/*--------searchVar-----------------------------------------------------
 Purpose:
 - Finds the given string, starting at the specified position

 NOTES:
 - If string is NULL, NULL is returned
----------------------------------------------------------------------*/
LPTSTR searchVar(
	LPCTSTR const string
)
{
	UINT i;

	i = 0;

	if( NULL != string && 0 != *string )
	{
		size_t len = _tcslen(string);

		while(
			i < g_mbf.nBufVAR
			&&
			!(
				0 == _tcsnicmp(g_mbf.pszBufVAR[i], string, len)
				&&
				(
					_istspace(*(g_mbf.pszBufVAR[i] + len))
					||
					0 == *(g_mbf.pszBufVAR[i] + len)
				)
			)
		) ++i;
	}
	else
	{
		i = g_mbf.nBufVAR;
	}

	return( i == g_mbf.nBufVAR ? NULL:g_mbf.pszBufVAR[i] );
}

VOID delVar(
	LPCTSTR const string
)
{
	UINT i;

	i = 0;

	if( NULL != string && 0 != *string )
	{
		size_t len = _tcslen(string);

		while(
			i < g_mbf.nBufVAR
			&&
			!(
				0 == _tcsnicmp(g_mbf.pszBufVAR[i], string, len)
				&&
				(
					_istspace(*(g_mbf.pszBufVAR[i] + len))
					||
					0 == *(g_mbf.pszBufVAR[i] + len)
				)
			)
		) ++i;
	}
	else
	{
		i = g_mbf.nBufVAR;
	}

	if( i != g_mbf.nBufVAR )
	{
		LPTSTR *tmpBuf;

		g_mbf.nBufVAR -= 1;

		if(NULL != g_mbf.pszBufVAR[i])
		{
			free(g_mbf.pszBufVAR[i]);
			g_mbf.pszBufVAR[i] = NULL;
		}

		if(i != g_mbf.nBufVAR)
		{
			memcpy(&g_mbf.pszBufVAR[i], &g_mbf.pszBufVAR[i+1], (g_mbf.nBufVAR - i) * sizeof(LPCTSTR));
		}

		tmpBuf = (LPTSTR*)realloc(
			g_mbf.pszBufVAR,
			g_mbf.nBufVAR * sizeof( LPTSTR )
		);

		if(NULL == tmpBuf)
		{
			g_mbf.pszBufVAR[g_mbf.nBufVAR] = NULL;
		}
		else
		{
			g_mbf.pszBufVAR = tmpBuf;
		}
	}

	return;
}

/*--------End Of File-------------------------------------------------*/
