/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2004 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Disable "unreferenced formal parameter" compiler warning
----------------------------------------------------------------------*/
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
/*---Shell Includes---------------------------------------------------*/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*--------------------------------------------------------------------*/


/*---Global Variables-------------------------------------------------*/
static LPCTSTR const s_szAppClass = _T("TApplication"); /* Our Window Class */
static LPCTSTR const s_szAppName  = _T("LiteStep");     /* Our Window Title */
static ATOM s_ClassAtom;

LPTSTR g_pszAppPath;
HINSTANCE g_hMainInst;
HWND g_hMainWnd;

BOOL g_bIgnoreModRetVal;


static HMODULE s_hShellsvc;
SS_Enum fpTrayEnum;

MS_WinShutdownFunc fpMSWinShutdown;
MS_RunFunc fpMSRun;

/* These must be exported by ordinal value for 9x compatibility */
FUNC_BOOL__HWND fpRegisterShellHookWindow;
FUNC_BOOL__HWND fpDeregisterShellHookWindow;
FUNC_VOID__HWND_BOOL fpSwitchToThisWindow;
/*--------------------------------------------------------------------*/


/*---Local Functions--------------------------------------------------*/
static VOID _trayUnregister(VOID);
static VOID _trayRegister(VOID);
static BOOL _checkSetup(LPCTSTR const);
static VOID _freeSetup(VOID);
static VOID _cleanExit(VOID);
static VOID _initAPI(VOID);
static INT _messageLoop(VOID);
/*--------------------------------------------------------------------*/


/*--------TrayCallBack--------------------------------------------------
 Purpose:
 - Callback registered with the Shell Service for recieving 
   notifications for the tray icons.
----------------------------------------------------------------------*/
BOOL CALLBACK TrayCallBack(
	 UINT uMsg
	,WPARAM wParam
	,LPARAM lParam
)
{
	if(NULL != g_hMainWnd)
	{
		SendMessage(g_hMainWnd, uMsg, wParam, lParam);
	}

	return(TRUE);
}


static VOID _trayUnregister(
	VOID
)
{
	if(NULL != s_hShellsvc)
	{
		SS_UnTrayRegister fpTrayUnRegister;

		fpTrayUnRegister = (SS_UnTrayRegister)GetProcAddress(
			 s_hShellsvc
			,"SS_UnregisterCallback"
		);

		if(NULL != fpTrayUnRegister)
		{
			fpTrayUnRegister(TrayCallBack);
		}

		FreeLibrary(s_hShellsvc);
		s_hShellsvc = NULL;
	}

	return;
}


static VOID _trayRegister(
	VOID
)
{
	_TCHAR tmpPath[1024] = {0};
	BOOL bSucceed = FALSE;

	/*------------------------------------------------------------------
	 Load systray service support only if it has already been loaded
	 into our process.  Then load our own copy so we aren't depending on
	 another module to keep the library loaded while we need it.
	------------------------------------------------------------------*/
	_stprintf(tmpPath, _T("%s\\%s"), g_pszAppPath, _T("shellsvc.dll"));

	if(
		NULL != GetModuleHandle(tmpPath)
		&&
		NULL != (s_hShellsvc = LoadLibrary(tmpPath))
	)
	{
		SS_TrayRegister fpTrayRegister;

		fpTrayRegister = (SS_TrayRegister)GetProcAddress(
			 s_hShellsvc
			,"SS_RegisterCallback"
		);

		fpTrayEnum = (SS_Enum)GetProcAddress(
			 s_hShellsvc
			,"SS_EnumNotifyIcons"
		);

		if(NULL != fpTrayRegister)
		{
			bSucceed = fpTrayRegister(TrayCallBack);
		}
	}

#ifdef _DEBUG
	if(!bSucceed)
	{
		ErrorMessageBox(_T("Error registering tray callback"), _T("Error"), MB_OK);
	}
#endif /* _DEBUG */

	return;
}


/*--------_checkSetup---------------------------------------------------
 Purpose:
 - Stores path we are being run from.
----------------------------------------------------------------------*/
static BOOL _checkSetup(
	LPCTSTR const pAppPath
)
{
	if(NULL != pAppPath && 0 != *pAppPath)
	{
		g_pszAppPath = (LPTSTR)malloc(
			(_tcslen(pAppPath) + 1) * sizeof(_TCHAR)
		);
	}

	if(NULL != g_pszAppPath)
	{
		_tcscpy(g_pszAppPath, pAppPath);
	}

	g_bIgnoreModRetVal = SH_CFG_GetBool(_T("LSModuleIgnoreFailure"));

	_initAPI();

	return(NULL != g_pszAppPath);
}


/*--------_freeSetup----------------------------------------------------
 Purpose:
 - Frees all resources allocated from _checkSetup()
----------------------------------------------------------------------*/
static VOID _freeSetup(
	VOID
)
{
	if(NULL != g_pszAppPath)
	{
		free(g_pszAppPath);
		g_pszAppPath = NULL;
	}

	return;
}


static VOID _cleanExit(
	VOID
)
{
	/* Release our registration to the systray services */
	_trayUnregister();

	/* Make sure the about dialog has been closed. */
	UnloadAboutDialog();

	/* Remove modules */
	RemoveModules();

	/* Some modules are lazy, and don't clean up */
	PurgeMessages();
	PurgeBCs();
	RemoveModuleDialogs();

	/* Free allocated resources */
	_freeSetup();

	if(NULL != g_hMainWnd)
	{
		DestroyWindow(g_hMainWnd);
	}

	if(0 != s_ClassAtom)
	{
		UnregisterClass(MAKEINTATOM(s_ClassAtom), g_hMainInst);
		s_ClassAtom = 0;
	}

	return;
}

static VOID _initAPI(
	VOID
)
{
	fpMSRun = (MS_RunFunc)GetProcAddress(
		 GetModuleHandle(_T("SHELL32.DLL"))
		,MAKEINTRESOURCEA(0x003D)
	);

	fpMSWinShutdown = (MS_WinShutdownFunc)GetProcAddress(
		 GetModuleHandle(_T("SHELL32.DLL"))
		,MAKEINTRESOURCEA(0x003C)
	);

	fpRegisterShellHookWindow = (FUNC_BOOL__HWND)GetProcAddress(
		 GetModuleHandle(_T("USER32"))
		,"RegisterShellHookWindow"
	);

	fpDeregisterShellHookWindow = (FUNC_BOOL__HWND)GetProcAddress(
		 GetModuleHandle(_T("USER32"))
		,"DeregisterShellHookWindow"
	);

	fpSwitchToThisWindow = (FUNC_VOID__HWND_BOOL)GetProcAddress(
		 GetModuleHandle(_T("USER32"))
		,"SwitchToThisWindow"
	);

	return;
}

static INT _messageLoop(
	VOID
)
{
	MSG Msg;
	INT nMsgErr;

	while(0 != (nMsgErr = GetMessage(&Msg, NULL, 0, 0)))
	{
		if(-1 == nMsgErr)
		{
#ifdef _DEBUG
			_TCHAR szErr[32];
			_stprintf(szErr, _T("GetMessage failed: 0x%08X"), GetLastError());
			OutputDebugString(szErr);
#endif
		}
		else if(NULL == Msg.hwnd)
		{
			if(PM_QUIT == Msg.message)
			{
				/*------------------------------------------------------
				 We must clean up before destroying our main window.
				 If we do not and a module was created with their
				 window as a child of the shell, then that child window
				 will be destroyed before the module is unloaded.
				------------------------------------------------------*/
				_trayUnregister();
				UnloadAboutDialog();
				RemoveModules();

				/*------------------------------------------------------
				 Now remove our shell window.
				------------------------------------------------------*/
				if(NULL != g_hMainWnd)
				{
					DestroyWindow(g_hMainWnd);
				}
				else
				{
					PostQuitMessage(0);
				}
			}
			else
			{
				LRESULT lResult;

				PassOnMessage(
					 Msg.message
					,Msg.wParam
					,Msg.lParam
					,TRUE
					,&lResult
				);
			}
		}
		else if(!IsModuleDialogMsg(&Msg))
		{
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
	}

	return(Msg.wParam);
}

INT WINAPI ShellMain(
	 LPCTSTR pAppPath
	,HANDLE hReadyEvent
)
{
	if(NULL != g_hMainInst && _checkSetup(pAppPath))
	{
		WNDCLASS wc;

		memset(&wc, 0, sizeof(WNDCLASS));
		wc.lpfnWndProc = (WNDPROC)WndProc;
		wc.hInstance = g_hMainInst;
		wc.lpszClassName = s_szAppClass;
		wc.style = CS_NOCLOSE;

		s_ClassAtom = RegisterClass(&wc);

		if(0 != s_ClassAtom)
		{
			g_hMainWnd = CreateWindowEx(
				 WS_EX_TOOLWINDOW
				,MAKEINTATOM(s_ClassAtom)
				,s_szAppName
				,WS_POPUP
				,0,0,0,0
				,(HWND)NULL
				,(HMENU)NULL
				,g_hMainInst
				,NULL
			);

			if(NULL != g_hMainWnd)
			{
				INT nRetVal = -1;

				/*------------------------------------------------------
				 Register systray callback
				------------------------------------------------------*/
				_trayRegister();

				/*------------------------------------------------------
				 Load our Modules
				------------------------------------------------------*/
				if(!AddModules())
				{
					WarningMessageBox(
						 _T("No LiteStep Modules were loaded!")
						,_T("PureLS - Warning")
						,MB_OK
					);

					// TEMPORARY UNTIL WE HAVE RECOVERY THREAD IMPLEMENTED
					PostThreadMessage(GetCurrentThreadId(), PM_QUIT, 0, 0);
				}

				/*------------------------------------------------------
				 Signal event so shell can continue loading
				------------------------------------------------------*/
				if(NULL != hReadyEvent)
				{
					SetEvent(hReadyEvent);
				}

				/*------------------------------------------------------
				 Sit here until we are ready to exit
				------------------------------------------------------*/
				nRetVal = _messageLoop();

				/*------------------------------------------------------
				 Call clean exit, as it is possible that we did not
				 exit cleanly in the message loop, or that we never
				 got into the message loop in the first place.  If
				 either is the case, our modules and window is still
				 valid and must be cleaned up.  Also, any other
				 clean up/ shutdown routines are placed in there.
				------------------------------------------------------*/
				_cleanExit();
				return(nRetVal);
			}
		}
	}

	ErrorMessageBox(
		 _T("Could not load LiteStep Module System")
		,_T("PureLS - Error!")
		,MB_OK
	);

	/* Do not set hReadyEvent since we did not load correctly */

	_cleanExit();
	return( -1 );
}

#ifdef PLS_EXCEPTION_HANDLER
/*--------ShellExceptionHandler-----------------------------------------
 Purpose:
 - This method gets called whenever an exception happens inside lsapi,
   and if it's one of the modules that caused the exception, show the
   user a message that so was the case, and then unload the module.
----------------------------------------------------------------------*/
BOOL CALLBACK ShellExceptionHandler(
	EXCEPTION_POINTERS*	pExceptionData,
	PLS_STACKTRACE*		pTrace
)
{
	ModuleData* pModule = 0; /* Litestep Module data */
	HMODULE hLsapi = GetModuleHandle(_T("lsapi"));

	OutputDebugString(_T("In LSExceptionHandler\r\n"));

	/*------------------------------------------------------------------
	 Find out which litestep module that is reponsible, if any.
	------------------------------------------------------------------*/
	while((pTrace != 0) &&
		  (hLsapi != pTrace->StackFrame.hModule) &&
		  ((pModule=GetModuleData(pTrace->StackFrame.hModule)) == 0))
	{
		pTrace = pTrace->pOuterLevel;
	}

	/*------------------------------------------------------------------
	 If there wasn't a module behind this error, we can't do anything.
	------------------------------------------------------------------*/
	if(!pTrace)
	{
		return FALSE;
	}

	/*------------------------------------------------------------------
	 If the module wasn't one of the loaded, but rather lsapi itself
	------------------------------------------------------------------*/
	if(pModule == 0)
	{
		/* TODO: Try to correct the error (it's our fault) */
		return FALSE;
	}

	/*------------------------------------------------------------------
	 But if there was a responsible module, we should do the following:
	 * Notify the user the module has gone bad.
	 * Terminate the module as nicely as possible.
	 * Jump back into _messageLoop to continue execution.
	------------------------------------------------------------------*/

	/*------------------------------------------------------------------
	 Notify the user...
	------------------------------------------------------------------*/
	ErrorMessageBox(_T("ERROR IN MODULE"), pModule->szFileName, MB_OK);


	/*------------------------------------------------------------------
	 If we can't stop the module, then the problem will still be there,
	 thus we haven't fixed the problem if the module is still loaded.
	------------------------------------------------------------------*/
	if (!StopModule(pModule->szFileName))
	{
		return FALSE;
	}

	/*------------------------------------------------------------------
	 Jump into the _messageLoop function
	------------------------------------------------------------------*/
	while((pTrace != 0) &&
		  (pTrace->StackFrame.ulFunctionAddress != (ULONG)_messageLoop))
	{
		pTrace = pTrace->pOuterLevel;
	}

	/*------------------------------------------------------------------
	 The _messageLoop function was not found, there's not so much to do.
	------------------------------------------------------------------*/
	if(!pTrace || !pTrace->pOuterLevel)
	{
		return FALSE;
	}

	/*------------------------------------------------------------------
	 I don't know if that will always be the case, but from what I've
	 seen a call to _messageLoop becomes the equivalent of:
	     __asm call _messageLoop
	 which in turn goes into the form:
	     0xE8 0xAA 0xBB 0xCC 0xDD
	 where, 0xDDCCBBAA is the byte offset to _messageLoop

	 This results in the fact that (ReturnAddress - 5) evaluates to the
	 address of the 0xE8 above, which means that the first thing that
	 will happen is _messageLoop being called again...
	------------------------------------------------------------------*/
	pExceptionData->ContextRecord->Eip =
		pTrace->StackFrame.ulReturnAddress - 5;

	/*------------------------------------------------------------------
	 When calling _messageLoop, the return value will be pushed onto
	 the stack. And then the first thing inside the routine is pushing
	 he base pointer aswell onto the stack. We want the stackpointer
	 s it was before the call, and therefore we add 2*4 to the
	 FramePointer from inside _messageLoop.
	 Actually, this would work even if _messageLoop had parameters.
	------------------------------------------------------------------*/
	pExceptionData->ContextRecord->Esp =
		pTrace->StackFrame.ulFramePointer + 2*sizeof(ULONG);

	/*------------------------------------------------------------------
	 The new base pointer we simply get by checking what it was before
	------------------------------------------------------------------*/
	pExceptionData->ContextRecord->Ebp =
		pTrace->pOuterLevel->StackFrame.ulFramePointer;

	/*------------------------------------------------------------------
	 If we get all the way here, the exception should be fixed properly
	 and thus we return TRUE to tell that we want execution to continue
	------------------------------------------------------------------*/
	return TRUE;
}
#endif /* PLS_EXCEPTION_HANDLER */

BOOL WINAPI DllMain(
	 HINSTANCE hinstDLL
	,DWORD fdwReason
	,LPVOID lpvReserved
)
{
	BOOL retVal = TRUE;

    switch(fdwReason)
    {
    case DLL_PROCESS_ATTACH:
		if(DisableThreadLibraryCalls(hinstDLL))
		{
			g_hMainInst = hinstDLL;
		}
		else
		{
			retVal = FALSE;
		}
		break;

	case DLL_PROCESS_DETACH:
		g_hMainInst = NULL;
		break;
	}

	return(retVal);
	UNREFERENCED_PARAMETER(lpvReserved);
}

/*--------End Of File-------------------------------------------------*/
