/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2004 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Disable "unreferenced formal parameter" compiler warning
----------------------------------------------------------------------*/
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
/*---Shell Includes---------------------------------------------------*/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*--------------------------------------------------------------------*/

/*---Local Definitions------------------------------------------------*/
#if defined(__MINGW32__)
static LPCTSTR const s_pszRevInfo = _T("0.40.1");
#else
static LPCTSTR const s_pszRevInfo = _T("0.40.1 ");// _T(__TIMESTAMP__);
#endif

typedef struct _HWNDITEM
{
	HWND hWnd;
	//
	struct _HWNDITEM * pNext;
	//
} HWNDITEM, *PHWNDITEM;

typedef struct _MESSAGEITEM
{
	UINT uMsg;
	PHWNDITEM pWndList;
	//
	struct _MESSAGEITEM * pNext;
	//
} MESSAGEITEM, *PMESSAGEITEM;
/*--------------------------------------------------------------------*/

/*---Global Variables-------------------------------------------------*/
extern FUNC_BOOL__HWND fpRegisterShellHookWindow;
extern FUNC_BOOL__HWND fpDeregisterShellHookWindow;

extern HWND g_hMainWnd;
extern UINT WM_ShellHook;
extern SS_Enum fpTrayEnum;

/*---Local Variables--------------------------------------------------*/
static PMESSAGEITEM s_pMsgList;
static UINT s_uShellHook;

/*---Local Functions--------------------------------------------------*/
static VOID _recycle(VOID);
static VOID _refresh(VOID);
/*--------------------------------------------------------------------*/



/*--------------------------------------------------------------------*/
static PMESSAGEITEM _getMessageItem( const UINT uMsg )
{
	PMESSAGEITEM pmi;

	for(pmi = s_pMsgList; NULL != pmi; pmi = pmi->pNext)
	{
		if(uMsg == pmi->uMsg)
		{
			break;
		}
	}

	return(pmi);
}

/*--------PassOnMessage-------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL PassOnMessage(
	 const UINT uMsg
	,const WPARAM wParam
	,const LPARAM lParam
	,const BOOL bWait
	,LRESULT * const pLResult
)
{
	PMESSAGEITEM pmi;

	pmi = _getMessageItem(uMsg);

	if(NULL != pmi)
	{
		PHWNDITEM pWnd;

		for(pWnd = pmi->pWndList; NULL != pWnd; pWnd = pWnd->pNext)
		{
			if(IsWindow(pWnd->hWnd))
			{
				if(bWait)
				{
					*pLResult |= SendMessage(pWnd->hWnd, uMsg, wParam, lParam);
				}
				else
				{
					PostMessage(pWnd->hWnd, uMsg, wParam, lParam);
				}
			}
		}

		return(TRUE);
	}

	return(FALSE);
}

/*--------------------------------------------------------------------*/
static VOID _addWindow( MESSAGEITEM * const pmi, const HWND hWnd )
{
	PHWNDITEM phi;

	for(phi = pmi->pWndList; NULL != phi; phi = phi->pNext)
	{
		if(hWnd == phi->hWnd)
		{
			break;
		}
	}

	if(NULL == phi)
	{
		phi = (PHWNDITEM)malloc(sizeof(HWNDITEM));

		if(NULL != phi)
		{
			phi->hWnd = hWnd;

			phi->pNext = pmi->pWndList;
			pmi->pWndList = phi;
		}
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _addMessage( const UINT uMsg, const HWND hWnd )
{
	PMESSAGEITEM pmi;

	pmi = _getMessageItem(uMsg);

	if(NULL == pmi)
	{
		pmi = (PMESSAGEITEM)malloc(sizeof(MESSAGEITEM));

		if(NULL != pmi)
		{
			pmi->uMsg = uMsg;
			pmi->pWndList = NULL;

			pmi->pNext = s_pMsgList;
			s_pMsgList = pmi;
		}
	}

	if(NULL != pmi)
	{
		_addWindow(pmi, hWnd);
	}

	return;
}

/*--------AddWndMessages------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID AddWndMessages( const UINT * puMsg, const HWND hWnd )
{
	if(NULL != puMsg && NULL != hWnd)
	{
		/* requires a terminating "0" zero value message */
		while(0 != *puMsg)
		{
			if(IsWindow(hWnd))
			{
				_addMessage(*puMsg++, hWnd);
			}
		}
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _removeWindow( MESSAGEITEM * const pMsgList, const HWND hWnd )
{
	PHWNDITEM pitem = pMsgList->pWndList, pprev = NULL;

	while(NULL != pitem)
	{
		if(hWnd == pitem->hWnd)
		{
			if(NULL == pprev)
			{
				pMsgList->pWndList = pitem->pNext;
			}
			else
			{
				pprev->pNext = pitem->pNext;
			}

			free(pitem);
			break;
		}

		pprev = pitem;
		pitem = pitem->pNext;
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _removeMessage( const UINT uMsg, const HWND hWnd )
{
	PMESSAGEITEM pitem = s_pMsgList, pprev = NULL;

	while(NULL != pitem)
	{
		if(uMsg == pitem->uMsg)
		{
			_removeWindow(pitem, hWnd);

			if(NULL == pitem->pWndList)
			{
				if(NULL == pprev)
				{
					s_pMsgList = pitem->pNext;
				}
				else
				{
					pprev->pNext = pitem->pNext;
				}

				free(pitem);
			}
			break;
		}

		pprev = pitem;
		pitem = pitem->pNext;
	}

	return;
}

/*--------RemoveWndMessages---------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID RemoveWndMessages( const UINT * puMsg, const HWND hWnd )
{
	if(NULL != puMsg && NULL != hWnd)
	{
		/* requires a terminating "0" zero value message */
		while(0 != *puMsg)
		{
			_removeMessage(*puMsg++, hWnd);
		}
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _freeMsgWndList( MESSAGEITEM * const pMsgList )
{
	PHWNDITEM phi;

	while(NULL != pMsgList->pWndList)
	{
		phi = pMsgList->pWndList->pNext;

		free(pMsgList->pWndList);

		pMsgList->pWndList = phi;
	}

	return;
}

/*--------PurgeMessages-------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID PurgeMessages( VOID )
{
	PMESSAGEITEM pmi;

	while(NULL != s_pMsgList)
	{
		pmi = s_pMsgList->pNext;

		_freeMsgWndList(s_pMsgList);
		free(s_pMsgList);

		s_pMsgList = pmi;
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _cleanDeadWindows( MESSAGEITEM * const pMsgList )
{
	PHWNDITEM pitem = pMsgList->pWndList, pprev = NULL, pnext;

	while(NULL != pitem)
	{
		pnext = pitem->pNext;

		if(!IsWindow(pitem->hWnd))
		{
			if(NULL == pprev)
			{
				pMsgList->pWndList = pnext;
			}
			else
			{
				pprev->pNext = pnext;
			}

			free(pitem);
		}
		else
		{
			pprev = pitem;
		}

		pitem = pnext;
	}

	return;
}

/*--------CleanDeadMessages---------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID CleanDeadMessages( VOID )
{
	PMESSAGEITEM pitem = s_pMsgList, pprev = NULL, pnext;

	while(NULL != pitem)
	{
		pnext = pitem->pNext;

		_cleanDeadWindows(pitem);

		if(NULL == pitem->pWndList)
		{
			if(NULL == pprev)
			{
				s_pMsgList = pnext;
			}
			else
			{
				pprev->pNext = pnext;
			}

			free(pitem);
		}
		else
		{
			pprev = pitem;
		}

		pitem = pnext;
	}

	return;
}

/*--------------------------------------------------------------------*/
static VOID _recycle(
	VOID
)
{
	UnloadAboutDialog();

	RemoveModules();
	/* Some modules are lazy, and don't clean up */
	PurgeMessages();
	PurgeBCs();
	RemoveModuleDialogs();

	SH_CFG_RefillBuffer();

	AddModules();

	return;
}

/*--------------------------------------------------------------------*/
static VOID _refresh(
	VOID
)
{
	SH_CFG_RefillBuffer();

	PassOnMessage(LM_REFRESH, 0, 0, FALSE, NULL);

	return;
}

/*--------LegacyLSMsg---------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL LegacyLSMsg( UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT *pLResult )
{
	switch(uMsg)
	{
	case LM_RECYCLE: /* Must be Posted */
		if (wParam == 0)
		{
			_recycle();
		}
		else if (wParam == 1)
		{
			/* Logoff */
			if (ExitWindowsEx(EWX_LOGOFF,0))
			{
				/* Some 9x systems won't close the shell automatically */
				PostThreadMessage(GetCurrentThreadId(), PM_QUIT, 0, 0);
			}
		}
		else if (wParam == 2)
		{
			/* quit */
			PostThreadMessage(GetCurrentThreadId(), PM_QUIT, 0, 0);
		}
		else
		{
			/* show shutdown dialog */
			BangShutdown(NULL, NULL);
		}
		break;

	case LM_SYSTRAYREADY: /* Must be Posted */
		if(NULL != fpTrayEnum)
		{
			fpTrayEnum(TrayCallBack);
		}
		break;

	case LM_REGISTERMESSAGE: /* Must be Sent */
		AddWndMessages((UINT*)lParam, (HWND)wParam);
		break;
	case LM_UNREGISTERMESSAGE: /* Must be Sent */
		RemoveWndMessages((UINT*)lParam, (HWND)wParam);
		break;

	case LM_UNLOADMODULE: /* Must be Sent */
		StopModule((LPCTSTR)wParam);
		break;
	case LM_RELOADMODULE: /* Must be Sent */
		StopModule((LPCTSTR)wParam);
		StartModule((LPCTSTR)wParam);
		break;

	case LM_BANGCOMMAND:
		{
			PLMBANGCOMMAND plmbc = (PLMBANGCOMMAND)lParam;

			if(plmbc->cbSize == sizeof(LMBANGCOMMAND))
			{
				*pLResult = ParseBangCommand(plmbc->hWnd, plmbc->szCommand, plmbc->szArgs);
			}
		}
		break;

	case LM_GETREVID: /* Must be Sent */
		//if(InSendMessage())
		{
			PMESSAGEITEM pmi;
			LPTSTR buf = (LPTSTR)lParam;
			INT bufLen = wParam >> 4;
			INT RetType = wParam & 0x0000f;

			if (RetType == 0)
			{
#ifdef _UNICODE
				_stprintf(buf, _T("PureLS-unicode: %s"), s_pszRevInfo);
#else
				_stprintf(buf, _T("PureLS: %s"), s_pszRevInfo);
#endif
			}
			else if (RetType == 1)
			{
#if 1
#ifdef _UNICODE
				_stprintf(
					 buf
					,_T("PureLS-unicode (www.purels.org): %s\n%s")
					,s_pszRevInfo
					,_T(" - purels.exe")
				);
#else /* not _UNICODE */
				_stprintf(
					 buf
					,_T("PureLS (www.purels.org): %s\n%s")
					,s_pszRevInfo
					,_T(" - purels.exe")
				);
#endif /* _UNICODE */
				if(NULL != GetModuleHandle(_T("config.dll")))
				{
					_tcscat(buf, _T("\n - config.dll"));
				}
				if(NULL != GetModuleHandle(_T("plsapi.dll")))
				{
					_tcscat(buf, _T("\n - plsapi.dll"));
				}
				if(NULL != GetModuleHandle(_T("shellsvc.dll")))
				{
					_tcscat(buf, _T("\n - shellsvc.dll"));
				}
				if(NULL != GetModuleHandle(_T("lsapi.dll")))
				{
					_tcscat(buf, _T("\n - lsapi.dll"));
				}
#else /* 1/0 */
#ifdef _UNICODE
				_stprintf(
					 buf
					,_T("PureLS-unicode (www.purels.org): %s\n%s\n%s\n%s\n%s\n%s")
					,s_pszRevInfo
					,_T(" - purels.exe")
					,_T(" - config.dll")
					,_T(" - plsapi.dll")
					,_T(" - shellsvc.dll")
					,_T(" - lsapi.dll")
				);
#else /* not _UNICODE */
				_stprintf(
					 buf
					,_T("PureLS (www.purels.org): %s\n%s\n%s\n%s\n%s\n%s")
					,s_pszRevInfo
					,_T(" - purels.exe")
					,_T(" - config.dll")
					,_T(" - plsapi.dll")
					,_T(" - shellsvc.dll")
					,_T(" - lsapi.dll")
				);
#endif /* _UNICODE */
#endif /* 1/0 */
			}
			else
			{
				*buf = 0;
			}

			pmi = _getMessageItem(uMsg);

			if(NULL != pmi)
			{
				_TCHAR buffer[1024];
				PHWNDITEM phi;

				if (RetType)
				{
					_tcscat(buf, _T("\n\nLoadModules:"));
				}

				for(phi = pmi->pWndList; NULL != phi; phi = phi->pNext)
				{
					*buffer = 0;

					if (
						SendMessage(
							 phi->hWnd
							,uMsg
							,(WPARAM)(wParam & 0x0F)
							,(LPARAM)buffer
						)
						&&
						0 != *buffer
					)
					{
						_tcscat(buf, _T("\n- "));
						_tcsncat(buf, buffer, bufLen - _tcslen(buf));
					}

					buf[bufLen-1] = 0;
				}
			}

			*pLResult = (LRESULT)_tcslen(buf);
		}
		break;

	default:
		return(FALSE);
	}

	return(TRUE);
}

/*--------LitestepMsgHdlr-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
BOOL LitestepMsgHdlr( UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT *pLResult )
{
	switch(uMsg)
	{
	/* handles events that will cause pLS to quit. */
	case PM_QUIT: /* Must be Posted */
		switch(wParam)
		{
		case QC_LOGOFF:
			if(ExitWindowsEx(EWX_LOGOFF, 0))
			{
				/* Some 9x systems won't close the shell automatically */
				PostThreadMessage(GetCurrentThreadId(), PM_QUIT, 0, 0);
			}
			break;

		case QC_REBOOT:
			ExitWindowsEx(EWX_REBOOT, 0);
			break;

		case QC_SHUTDOWN:
			ExitWindowsEx(EWX_SHUTDOWN, 0);
			break;

		case QC_SHUTDOWN_DIALOG:
			BangShutdown(NULL, NULL);
			break;

		default:
			/* quit */
			PostThreadMessage(GetCurrentThreadId(), PM_QUIT, 0, 0);
			break;
		}
		break;

	case PM_RECYCLE: /* Must be Posted */
		_recycle();
		break;

	case PM_REFRESH: /* Must be Posted */
		_refresh();
		break;

	case PM_REGISTERDIALOG:
		RegisterDialogWnd((HWND)wParam);
		break;
	case PM_UNREGISTERDIALOG:
		UnregisterDialogWnd((HWND)wParam);
		break;

	case LSAPI_ABOUTDIALOG:
		switch(wParam)
		{
		case AD_CMD_OPEN:
			LoadAboutDialog(lParam);
			break;

		case AD_CMD_CLOSE:
			UnloadAboutDialog();
			break;
		}
		break;

	default:
		return(
			LegacyLSMsg(uMsg, wParam, lParam, pLResult)
			||
			PassOnMessage(uMsg, wParam, lParam, TRUE, pLResult)
		);
	}

	return TRUE;
}

/*--------------------------------------------------------------------*/
static VOID _onShellHook( WPARAM wParam, LPARAM lParam )
{
#if 1
	PassOnMessage(
		 (LOWORD(wParam) & 0x7FFF) + LM_SHELL_HOOK
		,(WPARAM)lParam
		,(LPARAM)(LOWORD(wParam) & 0x8000)
		,FALSE
		,NULL
	);
#else
	switch(LOWORD(wParam))
	{
	case 1://HSHELL_WINDOWCREATED:
		PassOnMessage(LM_WINDOWCREATED, lParam, 0, FALSE, NULL);
		break;

	case 2://HSHELL_WINDOWDESTROYED:
		PassOnMessage(LM_WINDOWDESTROYED, lParam, 0, FALSE, NULL);
		break;

	case 3://HSHELL_ACTIVATESHELLWINDOW:
		PassOnMessage(LM_ACTIVATESHELLWINDOW, lParam, 0, FALSE, NULL);
		break;

	case 4://HSHELL_WINDOWACTIVATEED:
		PassOnMessage(LM_WINDOWACTIVATED, lParam, 0, FALSE, NULL);
		break;

	case 5://HSHELL_GETMINRECT:
		PassOnMessage(LM_GETMINRECT, lParam, 0, FALSE, NULL);
		break;

	case 6://HSHELL_REDRAW:
		PassOnMessage(LM_REDRAW, lParam, 0, FALSE, NULL);
		break;

	case 7://HSHELL_TASKMAN:
		PassOnMessage(LM_TASKMAN, lParam, 0, FALSE, NULL);
		break;

//	case 8://HSHELL_LANGUAGE:
//		PassOnMessage(LM_LANGUAGE, lParam, 0, FALSE, NULL);
//		break;

//	case 9://HSHELL_SYSMENU:
//		PassOnMessage(LM_SYSMENU, lParam, 0, FALSE, NULL);
//		break;

	case 10://HSHELL_ENDTASK:
		PassOnMessage(LM_ENDTASK, lParam, 0, FALSE, NULL);
		break;

//	case 11://HSHELL_ACCESSIBILITYSTATE:
//		PassOnMessage(LM_ACCESSIBILITYSTATE, lParam, 0, FALSE, NULL);
//		break;

	case 12://HSHELL_APPCOMMAND:
		PassOnMessage(LM_APPCOMMAND, lParam, 0, FALSE, NULL);
		break;

	case 13://HSHELL_WINDOWREPLACED:
		PassOnMessage(LM_WINDOWREPLACED, lParam, 0, FALSE, NULL);
		break;

	case 14://HSHELL_WINDOWREPLACING:
		PassOnMessage(LM_WINDOWREPLACING, lParam, 0, FALSE, NULL);
		break;

	case 6 + 0x8000://HSHELL_FLASH://HSHELL_REDRAW|HSHELL_HIGHBIT
		PassOnMessage(LM_REDRAW, lParam, HSHELL_HIGHBIT, FALSE, NULL);
		break;

	case 4 + 0x8000://HSHELL_RUDEAPPACTIVATEED://HSHELL_WINDOWACTIVATED|HSHELL_HIGHBIT
		PassOnMessage(LM_WINDOWACTIVATED, lParam, HSHELL_HIGHBIT, FALSE, NULL);
		break;
	}
#endif
	return;
}

/*--------WndProc-------------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	LRESULT lResult = 0;

	switch(uMsg)
	{
	case WM_CREATE:
		s_uShellHook = RegisterWindowMessage(_T("SHELLHOOK"));

		if(NULL != fpRegisterShellHookWindow)
		{
			fpRegisterShellHookWindow(hWnd);
		}
		break;

	case WM_DESTROY:
		if(NULL != fpDeregisterShellHookWindow)
		{
			fpDeregisterShellHookWindow(g_hMainWnd);
		}
		g_hMainWnd = NULL;
		PostQuitMessage(0);
		break;

	case WM_COPYDATA:
		switch(((PCOPYDATASTRUCT)lParam)->dwData)
		{
		case LM_BANGCOMMAND:
			lResult = SendMessage(
				 hWnd
				,LM_BANGCOMMAND
				,0
				,(LPARAM)((PCOPYDATASTRUCT)lParam)->lpData
			);
			break;
		}
		break;

	default:
		if(uMsg == s_uShellHook)
		{
			_onShellHook(wParam, lParam);
		}
		else if(!LitestepMsgHdlr(uMsg, wParam, lParam, &lResult))
		{
			lResult = DefWindowProc(hWnd, uMsg, wParam, lParam);
		}
		break;
	}

	return(lResult);
}

/*--------End Of File-------------------------------------------------*/
