/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2004 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
/*---Shell Includes---------------------------------------------------*/
#include "lsapi.h"
/*--------------------------------------------------------------------*/

extern FUNC_VOID__HWND_BOOL fpSwitchToThisWindow;

extern HINSTANCE g_hMainInst;
extern HWND g_hMainWnd;


/*---Local Variables--------------------------------------------------*/
static DWORD s_dwAboutThread;
static HANDLE s_hAboutThread;
static HWND s_hDlgWnd;
static HFONT s_hDlgFont;


/*---Local Defines----------------------------------------------------*/
#define ABOUT_WINDOW_TITLE   _T("About Information:")
#define ABOUT_BUFFER_SIZE    (2048)

#define ABOUT_CLOSE          (WM_USER + 1)
#define ABOUT_ACTIVATE       (WM_USER + 2)



#define ID_HELP   150
#define ID_TEXT   200

VOID ExitAboutDialog( VOID )
{
	DestroyWindow(s_hDlgWnd);
	PostMessage(g_hMainWnd, LSAPI_ABOUTDIALOG, AD_CMD_CLOSE, 0);

	return;
}

HFONT CreateSimpleFont( LPCTSTR faceName, INT sizeInPoints, BOOL bold, BOOL italic )
{
	// convert size from points to pixels
	LOGFONT lf;
	HDC hDC = GetDC( NULL );
	INT sizeInPixels = -MulDiv( sizeInPoints, GetDeviceCaps( hDC, LOGPIXELSY ), 72 );
	ReleaseDC( NULL, hDC );

	// fill in LOGFONT structure
	memset( &lf, 0, sizeof(LOGFONT) );

	lf.lfHeight = sizeInPixels;
	lf.lfWeight = bold ? FW_BOLD : FW_NORMAL;
	lf.lfItalic = (BYTE)italic;
	_tcsncpy( lf.lfFaceName, faceName, LF_FACESIZE );

	// create it
	return CreateFontIndirect( &lf );
}

INT_PTR CALLBACK DialogProc(
	HWND hWndDlg,
    UINT uMsg,
    WPARAM wParam,
    LPARAM lParam
)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		s_hDlgFont = CreateSimpleFont(_T("Tahoma"), 8, FALSE, FALSE);
		SendDlgItemMessage(hWndDlg, ID_TEXT, WM_SETFONT, (WPARAM)s_hDlgFont, FALSE);
		SendDlgItemMessage(hWndDlg, IDOK, WM_SETFONT, (WPARAM)s_hDlgFont, FALSE);
		break;

	case WM_DESTROY:
		s_hDlgWnd = NULL;
		PostQuitMessage(0);
		DeleteObject(s_hDlgFont);
		break;

	case WM_SYSCOMMAND:
		if(SC_CLOSE == (wParam & 0xFFF0))
		{
			ExitAboutDialog();
			break;
		}
		return(FALSE);

	case WM_COMMAND:
		if(IDOK == LOWORD(wParam))
		{
			ExitAboutDialog();
			break;
		}
		return(FALSE);

	default:
		return(FALSE);
	}

	return(TRUE);
	UNREFERENCED_PARAMETER(lParam);
}


LPWORD lpwAlign (LPBYTE lpIn)
{
	ULONG ul;

	ul = (ULONG) lpIn;

	ul  += 1;
	ul >>= 1;
	ul <<= 1;

	return (LPWORD) ul;
}

LPDWORD lpdwAlign (LPBYTE lpIn)
{
	ULONG ul;

	ul = (ULONG) lpIn;

	ul  += 3;
	ul >>= 2;
	ul <<= 2;

	return (LPDWORD) ul;
}

VOID GetDisplaySize(LPCTSTR const lpszMessage, short * const dwCX, short * const dwCY)
{
#if 0
	HDC hDC = GetDC( NULL );
	LPCTSTR pszPtr = lpszMessage;
	INT nSize = 0;
	SIZE size;

	while(0 != *pszPtr++)
	{
		nSize++;
	}

	GetTextExtentPoint32(hDC, lpszMessage, nSize+1, &size);

	ReleaseDC( NULL, hDC );

	*dwCX = (short)size.cx;
	*dwCY = (short)size.cy;

#else
	//HDC hDC = GetDC( NULL );
	INT sizeInPixelsY = 14;//MulDiv( 8, GetDeviceCaps( hDC, LOGPIXELSY ), 72 );
	INT sizeInPixelsX = 6;//MulDiv( 4, GetDeviceCaps( hDC, LOGPIXELSX ), 72 );
	LPCTSTR pszPtr = lpszMessage;
	INT nCountNewLines = 1;
	INT nMaxLineLength = 0;
	INT nLineLength = 0;

	//ReleaseDC( NULL, hDC );

	while(0 != *pszPtr)
	{
		if(_T('\n') == *pszPtr++)
		{
			nCountNewLines++;

			if(nMaxLineLength < nLineLength)
			{
				nMaxLineLength = nLineLength;
			}

			nLineLength = 0;
		}
		else
		{
			nLineLength++;
		}
	}

	*dwCX = (short)(nMaxLineLength * sizeInPixelsX);
	*dwCY = (short)(nCountNewLines * sizeInPixelsY);
#endif

	return;
}

HWND CreateAboutDialog(HINSTANCE hinst, HWND hwndOwner, LPCTSTR lpszMessage)
{
	HGLOBAL hgbl;
	LPDLGTEMPLATE lpdt;
	LPDLGITEMTEMPLATE lpdit;
	LPWORD lpw;
	LPDWORD lpdw;
	LPWSTR lpwsz;
	HWND hDlgWnd;
	int nchar;
	DWORD dwErr;
	short wCX, wCY, wX, wY;
	short wDlgCX, wDlgCY, wTextCX, wTextCY, wBtnCX, wBtnCY;
	RECT rWorkArea;

	hgbl = GlobalAlloc(GMEM_ZEROINIT, 1024 + (_tcslen(lpszMessage)+1) * 2);

	if(NULL == hgbl)
	{
		return NULL;
	}

	dwErr = GetDialogBaseUnits();

	wCX = LOWORD(dwErr);
	wCY = HIWORD(dwErr);

	SystemParametersInfo(SPI_GETWORKAREA, 0, &rWorkArea, 0);

	wX = (short)MulDiv(rWorkArea.right - rWorkArea.left, 4, wCX);
	wY = (short)MulDiv(rWorkArea.bottom - rWorkArea.top, 8, wCY);

	lpdt = (LPDLGTEMPLATE)GlobalLock(hgbl);

	// Define a dialog box.

	lpdt->style = WS_POPUPWINDOW | WS_CLIPSIBLINGS | WS_DLGFRAME | WS_OVERLAPPED | DS_SYSMODAL | DS_3DLOOK | DS_SETFOREGROUND;
	lpdt->dwExtendedStyle = WS_EX_TOPMOST | WS_EX_WINDOWEDGE;
	lpdt->cdit = 2;  // number of controls

	GetDisplaySize(lpszMessage, &wTextCX, &wTextCY);

	wTextCX = (short)MulDiv(wTextCX, 4, wCX);
	wTextCY = (short)MulDiv(wTextCY, 8, wCY);

	wBtnCX = (short)MulDiv(75, 4, wCX);
	wBtnCY = (short)MulDiv(23, 8, wCY);

	wDlgCY = (short)(wTextCY + MulDiv(GetSystemMetrics(SM_CYCAPTION), 8, wCY) + wBtnCY);
	wDlgCX = (short)(wTextCX + MulDiv(20, 4, wCX));

	lpdt->cx = wDlgCX;
	lpdt->cy = wDlgCY;

	lpdt->x  = (short)((wX - wDlgCX) / 2);
	lpdt->y = (short)((wY - wDlgCY) / 2);

	lpw = lpwAlign((LPBYTE)(lpdt + 1));
	*lpw++ = 0x0000;   // no menu
	*lpw++ = 0x0000;   // predefined dialog box class (by default)

	lpwsz = (LPWSTR)lpw;
	nchar = MultiByteToWideChar(CP_ACP, 0, "About Information:", -1, lpwsz, 50);
	lpw  += nchar;

	//-----------------------
	// Define an OK button.
	//-----------------------
	lpdw = lpdwAlign((LPBYTE)lpw); // align DLGITEMTEMPLATE on DWORD boundary
	lpdit = (LPDLGITEMTEMPLATE)lpdw;

	lpdit->cx = wBtnCX;
	lpdit->cy = wBtnCY;

	lpdit->x  = (WORD)((wDlgCX - wBtnCX) / 2);
	lpdit->y  = (WORD)(wDlgCY - wBtnCY);

	lpdit->id = IDOK;  // OK button identifier
	lpdit->style = WS_CHILDWINDOW | WS_VISIBLE | WS_GROUP | WS_TABSTOP | BS_DEFPUSHBUTTON | BS_TEXT;

	lpw = lpwAlign((LPBYTE)(lpdit + 1));
	*lpw++ = 0xFFFF;
	*lpw++ = 0x0080;    // button class

	lpwsz  = (LPWSTR)lpw;
	nchar  = MultiByteToWideChar (CP_ACP, 0, "OK", -1, lpwsz, 50);
	lpw   += nchar;

	*lpw++ = 0x0000;         // no creation data

	//-----------------------
	// Define a static text control.
	//-----------------------
	lpdw = lpdwAlign((LPBYTE)lpw); // align DLGITEMTEMPLATE on DWORD boundary
	lpdit = (LPDLGITEMTEMPLATE) lpdw;

	lpdit->cx = wTextCX;
	lpdit->cy = wTextCY;

	lpdit->x  = (short)((wDlgCX - wTextCX) / 2);
	lpdit->y  = 0;

	lpdit->id = ID_TEXT;  // text identifier
	lpdit->style = WS_CHILDWINDOW | WS_VISIBLE | WS_GROUP | SS_LEFT | SS_NOPREFIX | 0x00002000;

	lpw = lpwAlign((LPBYTE)(lpdit + 1));
	*lpw++ = 0xFFFF;
	*lpw++ = 0x0082;    // static class

	lpwsz  = (LPWSTR)lpw;
#ifndef _UNICODE
	nchar  = MultiByteToWideChar(CP_ACP, 0, lpszMessage, -1, lpwsz, strlen(lpszMessage)+50);
#else
	_tcscpy(lpwsz, lpszMessage);
	nchar = _tcslen(lpszMessage);
#endif
	lpw   += nchar;

	*lpw++ = 0x0000;         // no creation data

	GlobalUnlock(hgbl);

	hDlgWnd = CreateDialogIndirect(
		 hinst
		,(LPDLGTEMPLATE)hgbl
		,hwndOwner
		,(DLGPROC)DialogProc
	);

	dwErr = GetLastError();

	GlobalFree(hgbl);

	return hDlgWnd;
}

BOOL ShowAboutDialog( DWORD dwFlags )
{
	_TCHAR szBuffer[ABOUT_BUFFER_SIZE];
	DWORD dwType;

	dwType = (dwFlags & AD_TYPE_DETAIL) | ((ABOUT_BUFFER_SIZE - 1) << 4);

	if(0 != SendMessage(g_hMainWnd, LM_GETREVID, dwType, (LPARAM)szBuffer))
	{
		s_hDlgWnd = CreateAboutDialog(g_hMainInst, g_hMainWnd, szBuffer);

		if(NULL != s_hDlgWnd)
		{
			ShowWindow(s_hDlgWnd, SW_SHOWNORMAL);
		}
	}

	return(NULL != s_hDlgWnd);
}

/*--------RunAboutDialog------------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
UINT __stdcall RunAboutDialog(
	VOID * pVoid
)
{
	INT nMsgErr;
	MSG Msg;

	if(!ShowAboutDialog((DWORD)pVoid))
	{
		PostQuitMessage(0);
	}

	while(0 != (nMsgErr = GetMessage(&Msg, NULL, 0, 0)))
	{
		if(-1 == nMsgErr)
		{
		}
		else if(NULL == Msg.hwnd)
		{
			switch(Msg.message)
			{
			case ABOUT_ACTIVATE:
				if(NULL != s_hDlgWnd && NULL != fpSwitchToThisWindow)
				{
					fpSwitchToThisWindow(s_hDlgWnd, TRUE);
				}
				break;

			case ABOUT_CLOSE:
				if(NULL != s_hDlgWnd)
				{
					DestroyWindow(s_hDlgWnd);
				}
				break;

			default:
				break;
			}
		}
		else if(NULL != s_hDlgWnd)
		{
			if(!IsDialogMessage(s_hDlgWnd, &Msg))
			{
				TranslateMessage(&Msg);
				DispatchMessage(&Msg);
			}
		}
	}

	_endthreadex(0);
	return(0);
}

/*--------UnloadAboutDialog---------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID UnloadAboutDialog(
	VOID
)
{
	/*------------------------------------------------------------------
	 Verify Services Thread has finished
	------------------------------------------------------------------*/
	if(NULL != s_hAboutThread)
	{
		DWORD dwRet = STILL_ACTIVE;

		GetExitCodeThread(s_hAboutThread, &dwRet);

		/*--------------------------------------------------------------
		 Exit AboutDialog thread
		--------------------------------------------------------------*/
		if(STILL_ACTIVE == dwRet)
		{
			PostThreadMessage(s_dwAboutThread, ABOUT_CLOSE, 0, 0);
		}

#if 0
		WaitForSingleObject(s_hAboutThread, INFINITE);
#else
		/*--------------------------------------------------------------
		 Allow for a long 5 second timeout before nastily killing
		 the thread.  Hopefully it is never actually necessary.
		--------------------------------------------------------------*/
		dwRet = WaitForSingleObject(s_hAboutThread, 5000);
		if(WAIT_OBJECT_0 != dwRet)
		{
			TerminateThread(s_hAboutThread, 0);
		}
#endif
		CloseHandle(s_hAboutThread);
		s_hAboutThread = NULL;
	}

	return;
}

/*--------LoadAboutDialog-----------------------------------------------
 Purpose:
 -
----------------------------------------------------------------------*/
VOID LoadAboutDialog(
	DWORD dwFlags
)
{
	if(NULL != s_hAboutThread)
	{
		PostThreadMessage(s_dwAboutThread, ABOUT_ACTIVATE, 0, 0);
	}
	else
	{
		UINT uDummy;

		s_hAboutThread = (HANDLE)_beginthreadex(
			 NULL
			,0
			,RunAboutDialog
			,(VOID*)dwFlags
			,0
			,&uDummy
		);

		s_dwAboutThread = uDummy;
	}

	return;
}

/*--------End Of File-------------------------------------------------*/
