CC := gcc

# Some useful defines...
include Makefile.def

#------------------------------------------------------------------------
# Set directory where sources are to be found...
#------------------------------------------------------------------------
ifeq ($(srcdir),)
	$(targetname)sourcedir  := ../../$(targetname)
else
	$(targetname)sourcedir  := $(srcdir)/$(targetname)
endif
ifeq ($(srcdir),)
	$(targetname)libdir     := ../../sdk/mgwlib
else
	$(targetname)libdir     := $(srcdir)/sdk/mgwlib
endif

#------------------------------------------------------------------------
# Select settings depending on buildtype
#------------------------------------------------------------------------
ifeq ($(RELEASE),TRUE)
	outputdir := release
	CFLAGS    := $(CDEFINES) -Wall -D_NDEBUG -O3 -fomit-frame-pointer -mwindows -mthreads
	libending := 
else
	outputdir := debug
	CFLAGS    := $(CDEFINES) -D_DEBUG -Wall -pipe -mwindows -mthreads -g3
	libending := d
endif
CLFLAGS        := -Wl,--library-path=$($(targetname)libdir)

#------------------------------------------------------------------------
# The makefile including this only has to define:
#  targetname := the name of the target (like purels for instance)
#  targetfile := the name of the target file (like purels.exe)
#  targetreqs := the name(s) of the project which are prerequisits
#  targetlibs := the name(s) of the "common" importlibraries
#------------------------------------------------------------------------
$(targetname)temp       := $(targetname)_$(outputdir)
$(targetname)targetfile := $(outputdir)/$(targetfile)
$(targetname)sources    := $(wildcard $($(targetname)sourcedir)/*.c)
$(targetname)depends    := $(patsubst %.c,%.d,$(subst $($(targetname)sourcedir)/,$($(targetname)temp)/,$($(targetname)sources)))

#------------------------------------------------------------------------
# make sure that "make purels" is turned into "make debug/purels.exe"
#------------------------------------------------------------------------
$(targetname): $($(targetname)targetfile)
	@echo Build [$(targetname)] complete

#------------------------------------------------------------------------
# Target to builf an .exe file
#------------------------------------------------------------------------
$(outputdir)/%.exe: $($(targetname)depends) $(outputdir) \
                    $($(targetname)depends:.d=.o) \
                    $(targetreqs)
	@echo Creating $@
	@$(CC) $(CFLAGS) $(CLFLAGS) -o $@ \
	       $($(targetname)depends:.d=.o) \
	       $(foreach lib,$(targetlibs),-l$(lib)) \
	       $(foreach lib,$(targetreqs),-l$(lib)$(libending))

#------------------------------------------------------------------------
# Target to builf a .dll file
#------------------------------------------------------------------------
$(outputdir)/%.dll: $($(targetname)depends) $(outputdir) \
                    $($(targetname)depends:.d=.o) \
                    $(targetreqs)
	@echo Creating $@
	@$(CC) $(CFLAGS) $(CLFLAGS) -shared -o $@ \
	       -Wl,--add-stdcall-alias,--enable-stdcall-fixup \
	       -Wl,--out-implib,$($(targetname)libdir)/lib$(targetname)$(libending).a  \
	       $($(targetname)sourcedir)/exports.def \
	       $($(targetname)depends:.d=.o) \
	       $(foreach lib,$(targetlibs),-l$(lib)) \
	       $(foreach lib,$(targetreqs),-l$(lib)$(libending))
# gcc is faster&better at compiling and linking in one step...
	@$(if $(cygwin),rm -f,del /q/s/f) $($(targetname)depends:.d=.o)

$(targetreqs):
	@$(MAKE) --no-print-directory $@


include $($(targetname)depends)

#------------------------------------------------------------------------
# Simple helper targets to make sure folders are in place
#------------------------------------------------------------------------
$(outputdir) $($(targetname)temp) $($(targetname)libdir):
	@mkdir $@

# Make sure that cleaning is done even if there is a file named "clean"
.PHONY: clean

#------------------------------------------------------------------------
# Generate a name.d makefile with the dependencies for every name.c
#------------------------------------------------------------------------
$($(targetname)temp)/%.d: $($(targetname)sourcedir)/%.c
	@$(if $?,echo Generating dependencies from $<)
# prepend with file with own filename and a backslash
	@$(if $?,$(if $(cygwin), echo "$@ \\", echo $@ \ ) > $@)
	@$(if $?,$(CC) -MM -Wnone $< >> $@ || rm -f $@)
	        
#------------------------------------------------------------------------
# Compile a name.c into a name.o
#------------------------------------------------------------------------
$($(targetname)temp)/%.o: $($(targetname)sourcedir)/%.c
	@$(if $?,echo Compiling $?)
	@$(if $?,$(CC) $(CFLAGS) -o $@ -c $?)

#------------------------------------------------------------------------
# Make sure temp folder exists first...
#------------------------------------------------------------------------
$($(targetname)sourcedir)/%.c: $($(targetname)temp)
	
