/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <shellapi.h>
/*---Language Includes--------------------------------------------------------*/
#include <stdlib.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "shellsvc.h"
//#include "../sdk/include/purels.h"
/*----------------------------------------------------------------------------*/

EXTERN_C DECLSPEC_IMPORT PVOID STDAPICALLTYPE SHLockShared(HANDLE, DWORD);
EXTERN_C DECLSPEC_IMPORT BOOL STDAPICALLTYPE SHUnlockShared(PVOID);

extern VOID getTrayStubRect( LPRECT lpRect );

/*---Global Variables---------------------------------------------------------*/
extern CRITICAL_SECTION g_csAppBarList;
extern HWND s_hMainWnd;
/*----------------------------------------------------------------------------*/

/*---Local Variables----------------------------------------------------------*/
static PABDLINKLIST s_pItems;
/*----------------------------------------------------------------------------*/


#ifdef SHELLSVC_APPBAR_ENABLE


#ifdef LOG_APPBAR
#include <stdio.h>

extern FILE * fAppBarLog;

static void _logabd(const APPBARDATA * const pabd)
{
	_ftprintf(fAppBarLog, _T("\tAPPBARDATA (0x%08X)\n\t{\n"), pabd);

	_ftprintf(fAppBarLog, _T("\t\tDWORD  cbSize           = 0x%08X\n"), pabd->cbSize);
	_ftprintf(fAppBarLog, _T("\t\tHWND   hWnd             = 0x%08X\n"), pabd->hWnd);
	_ftprintf(fAppBarLog, _T("\t\tUINT   uCallbackMessage = 0x%08X\n"), pabd->uCallbackMessage);
	_ftprintf(fAppBarLog, _T("\t\tUINT   uEdge            = 0x%08X\n"), pabd->uEdge);

	_ftprintf(fAppBarLog, _T("\t\t\t("));
	switch(pabd->uEdge)
	{
	case ABE_BOTTOM:
		_ftprintf(fAppBarLog, _T(" ABE_BOTTOM"));
		break;
	case ABE_LEFT:
		_ftprintf(fAppBarLog, _T(" ABE_LEFT"));
		break;
	case ABE_RIGHT:
		_ftprintf(fAppBarLog, _T(" ABE_RIGHT"));
		break;
	case ABE_TOP:
		_ftprintf(fAppBarLog, _T(" ABE_TOP"));
		break;
	default:
		break;
	}

	_ftprintf(fAppBarLog, _T(")\n"));

	_ftprintf(fAppBarLog, _T("\t\tRECT   rc               = 0x%08X\n"), &pabd->rc);

	_ftprintf(fAppBarLog, _T("\t\t\t(%d,%d,%d,%d)\n"), pabd->rc.left, pabd->rc.top, pabd->rc.right, pabd->rc.bottom);

	_ftprintf(fAppBarLog, _T("\t\tLPARAM lParam           = 0x%08X\n"), pabd->lParam);

	_ftprintf(fAppBarLog, _T("\t}\n"));
	return;
}

static void _logsharedabd(PAPPBARDATA const pabd)
{
	_ftprintf(fAppBarLog, _T("\n"));

	_ftprintf(fAppBarLog, _T("================================================================================\n"));

	_ftprintf(fAppBarLog, _T("SHARED APPBARDATA FROM SHLockShared()\n"));

	if(NULL == pabd)
	{
		_ftprintf(fAppBarLog, _T("IS NULL!!!!\n"));
	}
	else
	{
		_logabd(pabd);
	}

	_ftprintf(fAppBarLog, _T("\n"));

	_ftprintf(fAppBarLog, _T("================================================================================\n\n"));

	fflush(fAppBarLog);

	return;
}

static void _logAppbar(PCSHELLAPPBARDATA const psad )
{
	if(NULL == fAppBarLog)
	{
		return;
	}

	_ftprintf(fAppBarLog, _T("============\n"));
	_ftprintf(fAppBarLog, _T("=AppBarMsg:=\n"));
	_ftprintf(fAppBarLog, _T("================================================================================\n"));

	_ftprintf(fAppBarLog, _T("dwMessage:"));
	switch(psad->dwMessage)
	{
	case ABM_ACTIVATE:
		_ftprintf(fAppBarLog, _T(" ABM_ACTIVATE"));
		break;
	case ABM_GETAUTOHIDEBAR:
		_ftprintf(fAppBarLog, _T(" ABM_GETAUTOHIDEBAR"));
		break;
	case ABM_GETSTATE:
		_ftprintf(fAppBarLog, _T(" ABM_GETSTATE"));
		break;
	case ABM_GETTASKBARPOS:
		_ftprintf(fAppBarLog, _T(" ABM_GETTASKBARPOS"));
		break;
	case ABM_NEW:
		_ftprintf(fAppBarLog, _T(" ABM_NEW"));
		break;
	case ABM_QUERYPOS:
		_ftprintf(fAppBarLog, _T(" ABM_QUERYPOS"));
		break;
	case ABM_REMOVE:
		_ftprintf(fAppBarLog, _T(" ABM_REMOVE"));
		break;
	case ABM_SETAUTOHIDEBAR:
		_ftprintf(fAppBarLog, _T(" ABM_SETAUTOHIDEBAR"));
		break;
	case ABM_SETPOS:
		_ftprintf(fAppBarLog, _T(" ABM_SETPOS"));
		break;
	case ABM_SETSTATE:
		_ftprintf(fAppBarLog, _T(" ABM_SETSTATE"));
		break;
	case ABM_WINDOWPOSCHANGED:
		_ftprintf(fAppBarLog, _T(" ABM_WINDOWPOSCHANGED"));
		break;
	}
	_ftprintf(fAppBarLog, _T("\n"));

	_ftprintf(fAppBarLog, _T("================================================================================\n"));

	_ftprintf(fAppBarLog, _T("SHELLAPPBARDATA\n{\n"));

	_logabd(psad->pabd);

	_ftprintf(fAppBarLog, _T("\tDWORD  dwMessage         = 0x%08X\n"), psad->dwMessage);
	_ftprintf(fAppBarLog, _T("\tHANDLE hSharedMemory     = 0x%08X\n"), psad->hSharedMemory);
	_ftprintf(fAppBarLog, _T("\tDWORD  dwSourceProcessId = 0x%08X\n"), psad->dwSourceProcessId);

	_ftprintf(fAppBarLog, _T("}\n"));
	_ftprintf(fAppBarLog, _T("================================================================================\n\n"));

	fflush(fAppBarLog);

	return;
}
#endif /* LOG_APPBAR */


/*--------_getListItem----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static PABDLINKLIST _getListItem(
	 HWND const hWnd
)
{
	PABDLINKLIST p = s_pItems;

	if(NULL == p)
	{
		p = s_pItems = (PABDLINKLIST)malloc(sizeof(ABDLINKLIST));

		if(NULL != p)
		{
			memset(p, 0, sizeof(ABDLINKLIST));

			p->abd.hWnd = hWnd;
		}
	}
	else
	{
		while(NULL != p)
		{
			if(p->abd.hWnd == hWnd)
			{
				break;
			}

			if(NULL == p->pNext)
			{
				p = p->pNext = (PABDLINKLIST)malloc(
					sizeof(ABDLINKLIST)
				);

				if(NULL != p)
				{
					memset(p, 0, sizeof(ABDLINKLIST));

					p->abd.hWnd = hWnd;
				}
				break;
			}

			p = p->pNext;
		}
	}

	return( p );
}

/*--------_findListItem---------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static PABDLINKLIST _findListItem(
	 HWND const hWnd
)
{
	PABDLINKLIST p = s_pItems;

	while(NULL != p)
	{
		if(hWnd == p->abd.hWnd)
		{
			break;
		}

		p = p->pNext;
	}

	return(p);
}


/*--------_delListItem----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
static BOOL _delListItem(
	PCABDLINKLIST const pll
)
{
	PABDLINKLIST p = s_pItems, pprev = NULL;

	while(NULL != p)
	{
		if(p == pll)
		{
			if(NULL == pprev)
			{
				s_pItems = p->pNext;
			}
			else
			{
				pprev->pNext = p->pNext;
			}

			free(p);

			return(TRUE);
		}

		pprev = p;
		p = p->pNext;
	}

	return(FALSE);
}

VOID raiseAutoHideWnds( VOID )
{
	PABDLINKLIST p = s_pItems;

	while(NULL != p)
	{
		if(p->bAutoHide)
		{
			SetWindowPos(p->abd.hWnd, HWND_TOP, 0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE|SWP_NOOWNERZORDER|SWP_NOSENDCHANGING);
		}

		p = p->pNext;
	}

	return;
}

HWND findAutoHideBar(UINT const uEdge)
{
	PABDLINKLIST p = s_pItems;
	HWND hWnd = NULL;

	while(NULL != p)
	{
		if(p->bAutoHide && uEdge == p->abd.uEdge)
		{
			hWnd = p->abd.hWnd;
			break;
		}

		p = p->pNext;
	}

	return(hWnd);
}

VOID notifyAppBar(DWORD dwMessage, UINT uEdge)
{
	PABDLINKLIST p = s_pItems;

	while(NULL != p)
	{
		if(uEdge == p->abd.uEdge)
		{
			SendMessage(p->abd.hWnd, p->abd.uCallbackMessage, dwMessage, 0);
		}

		p = p->pNext;
	}

	return;
}

VOID notifyPosChanged(HWND hSkipWnd, UINT uEdge)
{
	PABDLINKLIST p = s_pItems;
	BOOL bFound = NULL == hSkipWnd ? TRUE:FALSE;
	UINT uChkEdge = 0;

	while(NULL != p)
	{
		// Send a notification to the first appbar on a particular edge.
		// Except on our own edge, only send it to the first appbar after ours.
		if(0 == (uChkEdge & (1 << p->abd.uEdge)) && (bFound || uEdge != p->abd.uEdge))
		{
			uChkEdge |= 1 << p->abd.uEdge;
			SendMessage(p->abd.hWnd, p->abd.uCallbackMessage, ABN_POSCHANGED, 0);
		}

		if(hSkipWnd == p->abd.hWnd)
		{
			bFound = TRUE;
		}

		p = p->pNext;
	}

	return;
}

VOID modifyAppBar( PABDLINKLIST pItem, PAPPBARDATA pabd )
{
	RECT dtRect;
	PABDLINKLIST p = s_pItems;
	BOOL bFound = FALSE;

	GetWindowRect(GetDesktopWindow(), &dtRect);

	if(!pItem->bDocked)
	{
		while(NULL != p)
		{
			if(p->bDocked)
			{ // ignore docked
			}
			if(pabd->hWnd == p->abd.hWnd)
			{ // if this is our appbar, mark it found and continue
				bFound = TRUE;
			}
			else if(pabd->uEdge != p->abd.uEdge)
			{ // Ignore other edge appbars for the most part
				// Left/Right bars must resize to top/bottom bars
				if(
					ABE_HORIZONTAL != (ABE_HORIZONTAL & pabd->uEdge)
					&&
					ABE_HORIZONTAL == (ABE_HORIZONTAL & p->abd.uEdge)
				)
				{
					RECT rc; // dummy

					// If our destination rectangle currently intersects
					// then resize it
					if(IntersectRect(&rc, &(p->abd.rc), &dtRect))
					{
						if(ABE_TOP == p->abd.uEdge)
						{
							dtRect.top = p->abd.rc.bottom;
						}
						else
						{
							dtRect.bottom = p->abd.rc.top;
						}
					}
				}
			}
			else if(!bFound)
			{
				// Keep moving the destination appbar while we are looking
				// at same edge appbars before our own.  Once we find our
				// own, then we ignore all same edge appbars after ours.
				switch(pabd->uEdge)
				{
				case ABE_LEFT:
					dtRect.left = p->abd.rc.right;
					break;
				case ABE_TOP:
					dtRect.top = p->abd.rc.bottom;
					break;
				case ABE_RIGHT:
					dtRect.right = p->abd.rc.left;
					break;
				case ABE_BOTTOM:
					dtRect.bottom = p->abd.rc.top;
					break;
				}
			}

			p = p->pNext;
		}
	}

	switch(pabd->uEdge)
	{
	case ABE_LEFT:
		dtRect.right = dtRect.left + (pabd->rc.right - pabd->rc.left);
		break;
	case ABE_TOP:
		dtRect.bottom = dtRect.top + (pabd->rc.bottom - pabd->rc.top);
		break;
	case ABE_RIGHT:
		dtRect.left = dtRect.right + (pabd->rc.right - pabd->rc.left);
		break;
	case ABE_BOTTOM:
		dtRect.top = dtRect.bottom + (pabd->rc.bottom - pabd->rc.top);
		break;
	}

	CopyRect(&pabd->rc, &dtRect);

	return;
}


static LRESULT appBarHdlr(PCSHELLAPPBARDATA const psad)
{
	LRESULT lResult = 0;
	PAPPBARDATA pabd = NULL;

	PABDLINKLIST pItem;

	/*--------------------------------------------------------------------------
	 Find item being updated/removed, or create a new item
	--------------------------------------------------------------------------*/
	pItem = _findListItem(psad->pabd->hWnd);

	if(NULL != psad->hSharedMemory)
	{
		/* Happens for messages ABM_QUERYPOS, ABM_SETPOS, ABM_GETTASKBARPOS */
		pabd = (PAPPBARDATA)SHLockShared(psad->hSharedMemory, psad->dwSourceProcessId);

#ifdef LOG_APPBAR
		_logsharedabd(pabd);
#endif
	}

	switch(psad->dwMessage)
	{
	case ABM_ACTIVATE:
	case ABM_WINDOWPOSCHANGED:
		raiseAutoHideWnds();
		lResult = 1;
		break;

	case ABM_GETAUTOHIDEBAR:
		if(NULL != pItem)
		{
			lResult = (LRESULT)findAutoHideBar(psad->pabd->uEdge);
		}
		break;

	case ABM_GETSTATE:
		lResult = ABS_ALWAYSONTOP;//|ABS_AUTOHIDE;
		break;

	case ABM_GETTASKBARPOS:
		if(NULL != pabd)
		{
			getTrayStubRect(&pabd->rc);
			lResult = 1;
		}
		break;

	case ABM_NEW:
		if(NULL == pItem && IsWindow(psad->pabd->hWnd))
		{
			pItem = _getListItem(psad->pabd->hWnd);

			if(NULL != pItem)
			{
				pItem->abd.cbSize = sizeof(APPBARDATA);
				pItem->abd.hWnd = psad->pabd->hWnd;
				pItem->abd.uCallbackMessage = psad->pabd->uCallbackMessage;
				pItem->abd.uEdge = (UINT)-1;

				lResult = 1;
			}
		}
		break;

	case ABM_QUERYPOS:
		if(NULL != pabd && NULL != pItem)
		{
			modifyAppBar(pItem, pabd);
			lResult = 1;
		}
		break;

	case ABM_REMOVE:
		if(NULL != pItem)
		{
			PostMessage(
				 s_hMainWnd
				,ABP_NOTIFYPOSCHANGED
				,(WPARAM)pItem->abd.hWnd
				,(LPARAM)MAKELONG(1, pItem->abd.uEdge)
			);
			_delListItem(pItem);
		}

		lResult = 1;
		break;

	case ABM_SETAUTOHIDEBAR:
		if(psad->pabd->lParam) // Set Auto Hide
		{
			HWND hAutoHideWnd;

			hAutoHideWnd = findAutoHideBar(psad->pabd->uEdge);

			if(NULL != hAutoHideWnd)
			{
				lResult = (hAutoHideWnd == psad->pabd->hWnd) ? 1:0;
				break;
			}

			if(NULL == pItem)
			{
				pItem = _getListItem(psad->pabd->hWnd);

				if(NULL != pItem)
				{
					pItem->abd.cbSize = sizeof(APPBARDATA);
					pItem->abd.hWnd = psad->pabd->hWnd;
					pItem->abd.uCallbackMessage = psad->pabd->uCallbackMessage;
					pItem->bDocked = TRUE;
				}
			}

			if(NULL != pItem)
			{
				pItem->abd.uEdge = psad->pabd->uEdge;
				pItem->bAutoHide = TRUE;

				lResult = 1;
			}
		}
		else // Clear Auto Hide
		{
			if(NULL != pItem)
			{
				if(findAutoHideBar(psad->pabd->uEdge) == pItem->abd.hWnd)
				{
					lResult = 1;

					if(pItem->bDocked)
					{
						_delListItem(pItem);
						break;
					}
					else
					{
						pItem->bAutoHide = FALSE;
					}
				}
			}
		}

		if(lResult && !pItem->bDocked)
		{
			PostMessage(
				 s_hMainWnd
				,ABP_NOTIFYPOSCHANGED
				,(WPARAM)pItem->abd.hWnd
				,(LPARAM)MAKELONG(1, pItem->abd.uEdge)
			);
		}
		break;

	case ABM_SETPOS:
		if(NULL != pabd && NULL != pItem)
		{
			modifyAppBar(pItem, pabd);

			if(!pItem->bDocked && !EqualRect(&(pItem->abd.rc), &(pabd->rc)))
			{
				PostMessage(
					 s_hMainWnd
					,ABP_NOTIFYPOSCHANGED
					,(WPARAM)pItem->abd.hWnd
					,(LPARAM)MAKELONG(1, pItem->abd.uEdge)
				);
			}

			pItem->abd.uEdge = pabd->uEdge;
			CopyRect(&(pItem->abd.rc), &(pabd->rc));

			lResult = 1;
		}
		break;

	case ABM_SETSTATE:
		lResult = 1;
		break;

	default:
		break;
	}

	if(NULL != pabd)
	{
		SHUnlockShared(pabd);
	}

	return(lResult);
}

/*--------onAppbarMsg-----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
VOID onAppbarNotice( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	EnterCriticalSection(&g_csAppBarList);

	switch(uMsg)
	{
	case ABP_NOTIFYPOSCHANGED:
		notifyPosChanged((HWND)wParam, (UINT)lParam);
		break;
	case ABP_NOTIFYSTATECHANGE:
		break;
	case ABP_RAISEAUTOHIDEHWND:
		break;
	}

	LeaveCriticalSection(&g_csAppBarList);

	return;
}

/*--------onAppbarMsg-----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
LRESULT onAppbarMsg( PVOID lpData, DWORD const cbData )
{
	SHELLAPPBARDATA sad = {0};
	LRESULT lResult = 0;
	DWORD dwABSize = 0;

	switch(cbData)
	{
	case 0x38: //XP
		sad.pabd = &((PCAPPBARMSGDATAXP)lpData)->abd;
		sad.dwMessage = ((PCAPPBARMSGDATAXP)lpData)->dwMessage;
		sad.hSharedMemory = ((PCAPPBARMSGDATAXP)lpData)->hSharedMemory;
		sad.dwSourceProcessId = ((PCAPPBARMSGDATAXP)lpData)->dwSourceProcessId;
		dwABSize = 0x28;
		break;

	case 0x30: //2K //98se
		sad.pabd = &((PCAPPBARMSGDATA2K)lpData)->abd;
		sad.dwMessage = ((PCAPPBARMSGDATA2K)lpData)->dwMessage;
		sad.hSharedMemory = ((PCAPPBARMSGDATA2K)lpData)->hSharedMemory;
		sad.dwSourceProcessId = ((PCAPPBARMSGDATA2K)lpData)->dwSourceProcessId;
		dwABSize = 0x24;
		break;

	default:
		break;
	}

	if(NULL != sad.pabd && dwABSize == sad.pabd->cbSize)
	{
#ifdef LOG_APPBAR
		_logAppbar(&sad);
#endif

		EnterCriticalSection(&g_csAppBarList);

		lResult = appBarHdlr(&sad);

		LeaveCriticalSection(&g_csAppBarList);
	}

	return(lResult);
}

/*--------freeAppBarList--------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
VOID freeAppBarList(
	VOID
)
{
	PABDLINKLIST p;

	EnterCriticalSection(&g_csAppBarList);

	while(NULL != s_pItems)
	{
		p = s_pItems->pNext;

		free(s_pItems);

		s_pItems = p;
	}

	LeaveCriticalSection(&g_csAppBarList);

	return;
}

#endif /* SHELLSVC_APPBAR_ENABLE */

/*--------End Of File---------------------------------------------------------*/
