/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2002 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/

#ifndef __SHELLSVC_H
#define __SHELLSVC_H

	#define SHELLTRAY __declspec( dllexport )
	#include "../sdk/include/shelltray.h"

	#define STARTUP __declspec( dllexport )
	#include "../sdk/include/startup.h"

#if defined(__MINGW32__)
typedef enum RESTRICTIONS
{
	REST_NOLOCALMACHINERUNONCE      = 0x40000048,
	REST_NOLOCALMACHINERUN          = 0x40000046,
	REST_NOCURRENTUSERRUN           = 0x40000047,
	REST_NOCURRENTUSERRUNONCE       = 0x40000049,

} RESTRICTIONS;
#endif /* RESTRICTIONS */

	typedef DWORD (WINAPI* MS_SHRestricted)(RESTRICTIONS);

/*--------------------------------------------------------------------*/
#define SHELLSVC_APPBAR_ENABLE
//#define SHELLSVC_LOADINPROC_ENABLE

//#define LOG_STUFF
#if defined(_DEBUG) || defined(LOG_STUFF)
#define LOG_WM_COPYDATA
#define LOG_NOTIFYICON
#define LOG_APPBAR
#endif

/*--------------------------------------------------------------------*/

	// shell copy data types
	#define SH_APPBAR_MSG_DATA  (0)
	#define SH_NOTIFY_ICON_DATA (1)
	#define SH_LOAD_PROC_DATA   (2)

	// internally posted AppBar messages
	#define ABP_NOTIFYPOSCHANGED   (WM_USER+350)
	#define ABP_NOTIFYSTATECHANGE  (WM_USER+351)
	#define ABP_RAISEAUTOHIDEHWND  (WM_USER+360)

/*--------------------------------------------------------------------*/
	/* Defines which are normally only available if  _WIN32_IE >= 0x0500 and 0x0600 */
	#ifndef NIM_SETFOCUS
	#define NIM_SETFOCUS    0x00000003
	#endif
	#ifndef NIM_SETVERSION
	#define NIM_SETVERSION  0x00000004
	#endif

	#ifndef NIF_STATE
	#define NIF_STATE       0x00000008
	#endif
	#ifndef NIF_INFO
	#define NIF_INFO        0x00000010
	#endif
	#ifndef NIF_GUID
	#define NIF_GUID        0x00000020
	#endif

	#ifndef NIS_HIDDEN
	#define NIS_HIDDEN		0x00000001
	#endif
	#ifndef NIS_SHAREDICON
	#define NIS_SHAREDICON  0x00000002
	#endif

	#ifndef NIIF_NONE
	#define NIIF_NONE       0x00000000
	#endif
	#ifndef NIIF_INFO
	#define NIIF_INFO       0x00000001
	#endif
	#ifndef NIIF_WARNING
	#define NIIF_WARNING    0x00000002
	#endif
	#ifndef NIIF_ERROR
	#define NIIF_ERROR      0x00000003
	#endif
	#ifndef NIIF_ICON_MASK
	#define NIIF_ICON_MASK  0x0000000F
	#endif
	#ifndef NIIF_NOSOUND
	#define NIIF_NOSOUND    0x00000010
	#endif

/*--------------------------------------------------------------------*/
	/* the structure of notification icons in win9x */
	typedef struct _NOTIFYICONDATA9X
	{
		DWORD cbSize;
		HWND  hWnd;
		UINT  uID;
		UINT  uFlags;
		UINT  uCallbackMessage;
		HICON hIcon;
		CHAR  szTip[64];
	} NOTIFYICONDATA9X, *PNOTIFYICONDATA9X;

	/* the structure of notification icons in winNT... */
	typedef struct _NOTIFYICONDATANT
	{
		DWORD cbSize;
		HWND  hWnd;
		UINT  uID;
		UINT  uFlags;
		UINT  uCallbackMessage;
		HICON hIcon;
		WCHAR szTip[64];
	} NOTIFYICONDATANT, *PNOTIFYICONDATANT;

	/* the structure of notification icons in win2K... */
	typedef struct _NOTIFYICONDATA2K
	{
		DWORD cbSize;
		HWND  hWnd;
		UINT  uID;
		UINT  uFlags;
		UINT  uCallbackMessage;
		HICON hIcon;
		WCHAR szTip[128];
		DWORD dwState;
		DWORD dwStateMask;
		WCHAR szInfo[256];

		union
		{
			UINT uTimeout;
			UINT uVersion;
		} DUMMYUNIONNAME;

		WCHAR szInfoTitle[64];
		DWORD dwInfoFlags;
	} NOTIFYICONDATA2K, *PNOTIFYICONDATA2K;

	/* the structure of notification icons in winXP... */
	typedef struct _NOTIFYICONDATAXP
	{
		DWORD cbSize;
		HWND  hWnd;
		UINT  uID;
		UINT  uFlags;
		UINT  uCallbackMessage;
		HICON hIcon;
		WCHAR szTip[128];
		DWORD dwState;
		DWORD dwStateMask;
		WCHAR szInfo[256];

		union
		{
			UINT uTimeout;
			UINT uVersion;
		} DUMMYUNIONNAME;

		WCHAR szInfoTitle[64];
		DWORD dwInfoFlags;

		GUID guidItem;
	} NOTIFYICONDATAXP, *PNOTIFYICONDATAXP;

	#define IS2KXPHIDDENICON(x) (NIS_HIDDEN == ((x).dwStateMask & (x).dwState & NIS_HIDDEN))
	#define IS2KXPSHAREDICON(x) (NIS_SHAREDICON == ((x).dwStateMask & (x).dwState & NIS_SHAREDICON))

	/* Data format of incoming notifications */
	typedef struct _SHELLTRAYDATA
	{
		DWORD dwUnknown;
		DWORD dwMessage;
		union nid
		{
			NOTIFYICONDATA9X w9X;
			NOTIFYICONDATANT wNT;
			NOTIFYICONDATA2K w2K;
			NOTIFYICONDATAXP wXP;
		} nid;
	} SHELLTRAYDATA, *PSHELLTRAYDATA;
	typedef const SHELLTRAYDATA * PCSHELLTRAYDATA;


	/* XP */
	typedef struct _APPBARMSGDATAXP
	{
		APPBARDATA abd;
		/**/
		DWORD dwPadding;
		/**/
		DWORD  dwMessage;
		HANDLE hSharedMemory;
		DWORD  dwSourceProcessId;
		/**/
		DWORD dwPadding2;
		/**/
	} APPBARMSGDATAXP, *PAPPBARMSGDATAXP;
	typedef const APPBARMSGDATAXP * PCAPPBARMSGDATAXP;

	/* 2K */
	typedef struct _APPBARMSGDATA2K
	{
		APPBARDATA abd;
		/**/
		DWORD  dwMessage;
		HANDLE hSharedMemory;
		DWORD  dwSourceProcessId;
		/**/
	} APPBARMSGDATA2K, *PAPPBARMSGDATA2K;
	typedef const APPBARMSGDATA2K * PCAPPBARMSGDATA2K;

	/* SHELL */
	typedef struct _SHELLAPPBARDATA
	{
		const APPBARDATA * pabd;
		/**/
		DWORD  dwMessage;
		HANDLE hSharedMemory;
		DWORD  dwSourceProcessId;
		/**/
	} SHELLAPPBARDATA, *PSHELLAPPBARDATA;
	typedef const SHELLAPPBARDATA * PCSHELLAPPBARDATA;



	/* Used for internal storage of notification data */
	typedef struct _NIDLINKLIST
	{
		HWND  hWnd;
		UINT  uID;
		UINT  uFlags;
		UINT  uMesg;
		HICON hIcon;
		_TCHAR szTip[128];
		/**/
		struct _NIDLINKLIST *pNext;
		/**/
	} NIDLINKLIST, *PNIDLINKLIST;
	typedef const NIDLINKLIST * PCNIDLINKLIST;

	/* Used for link list of registered callbacks for the tray notifications */
	typedef struct _TEPLINKLIST
	{
		SHTRAYCALLBACKPROC   pProc;
		/**/
		struct _TEPLINKLIST *pNext;
		/**/
	} TEPLINKLIST, *PTEPLINKLIST;


	/* Used for AppBars */
	typedef struct _ABDLINKLIST
	{
		APPBARDATA abd;
		BOOL bAutoHide;
		BOOL bDocked;
		/**/
		struct _ABDLINKLIST *pNext;
		/**/
	} ABDLINKLIST, *PABDLINKLIST;
	typedef const ABDLINKLIST * PCABDLINKLIST;

	#define ABE_HORIZONTAL      0x01

/*--------------------------------------------------------------------*/

#endif /* __SHELLSVC_H */

