/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <shellapi.h>
#include <shlobj.h>
#include <io.h>
#include <stdio.h>
#include <regstr.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "shellsvc.h"
/*----------------------------------------------------------------------------*/


/*---Local Defines------------------------------------------------------------*/
#define REGSTR_PATH_POLICIES_RUN    REGSTR_PATH_POLICIES _T("\\Explorer\\Run")
#define USE_STARTUP_POLICIES
/*----------------------------------------------------------------------------*/


/*--------HasStartupBeenRun-----------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
BOOL WINAPI StartupHasBeenRun(
	VOID
)
{
	BOOL bRetVal = FALSE;

	OSVERSIONINFO OsVersionInfo;

	OsVersionInfo.dwOSVersionInfoSize = sizeof(OsVersionInfo);
	GetVersionEx(&OsVersionInfo);

	/* For NT the SessionInfo subkey will be the AuthenticationID */
	if( VER_PLATFORM_WIN32_NT == OsVersionInfo.dwPlatformId )
	{
		HANDLE hToken;

		if(OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
		{
			TOKEN_STATISTICS tsStats;
			DWORD dwOutSize;

			if(
				GetTokenInformation(
					 hToken
					,TokenStatistics
					,&tsStats
					,sizeof(tsStats)
					,&dwOutSize
				)
			)
			{
				HKEY hkExplorer;
				LONG lResult;

				/* Open Explorer subkey */
				lResult = RegCreateKeyEx(
					 HKEY_CURRENT_USER
					,REGSTR_PATH_EXPLORER
					,0
					,NULL
					,REG_OPTION_NON_VOLATILE
					,MAXIMUM_ALLOWED
					,NULL
					,&hkExplorer
					,NULL
				);

				/* Create the SessionInfo and StartUpHasBeenRun keys */
				if( ERROR_SUCCESS == lResult )
				{
					TCHAR tzSessionInfo[30];
					HKEY hkSessionInfo;

					_stprintf(
						 tzSessionInfo
						,_T("SessionInfo\\%08x%08x")
						,tsStats.AuthenticationId.HighPart
						,tsStats.AuthenticationId.LowPart
					);

					lResult = RegCreateKeyEx(
						 hkExplorer
						,tzSessionInfo
						,0
						,NULL
						,REG_OPTION_VOLATILE
						,KEY_WRITE
						,NULL
						,&hkSessionInfo
						,NULL
					);

					if( ERROR_SUCCESS == lResult )
					{
						DWORD dwDisposition;
						HKEY hkStartup;

						lResult = RegCreateKeyEx(
							 hkSessionInfo
							,_T("StartupHasBeenRun")
							,0
							,NULL
							,REG_OPTION_VOLATILE
							,KEY_WRITE
							,NULL
							,&hkStartup
							,&dwDisposition
						);

						RegCloseKey(hkStartup);

						if( REG_CREATED_NEW_KEY != dwDisposition )
						{
							bRetVal = TRUE;
						}
					}

					RegCloseKey(hkSessionInfo);
				}

				RegCloseKey(hkExplorer);
			}
		}

		CloseHandle(hToken);
	}

	return( bRetVal );
}


/*--------RunShellItem----------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID WINAPI RunShellItem(
	 LPCTSTR pszDir
	,LPCTSTR pszPath
	,LPCTSTR pszParams
)
{
	/*--------------------------------------------------------------------------
	 Lets make sure parameters are valid
	--------------------------------------------------------------------------*/
	if( NULL != pszPath && 0 != *pszPath )
	{
		SHELLEXECUTEINFO si;
		_TCHAR szDir[MAX_PATH] = {0};

		/*----------------------------------------------------------------------
		 Lets create our default folder from the filename if pszDir is NULL
		----------------------------------------------------------------------*/
		if( NULL == pszDir && NULL != _tcschr(pszPath, _T('\\')) )
		{
			UINT uFileLen = _tcslen(pszPath);

			while( 0 != uFileLen && pszPath[uFileLen-1] != _T('\\') )
			{
				--uFileLen;
			}

			if( 0 != uFileLen && uFileLen < MAX_PATH )
			{
				pszDir = _tcsncpy(szDir, pszPath, uFileLen);
				szDir[uFileLen] = 0;
			}
		}

		/*----------------------------------------------------------------------
		 Setup and execute the app
		----------------------------------------------------------------------*/
		memset(&si, 0, sizeof(SHELLEXECUTEINFO));
		si.cbSize = sizeof(SHELLEXECUTEINFO);
		/* use SEE_MASK_FLAG_DDEWAIT in si.fMask ? */
		si.fMask = SEE_MASK_DOENVSUBST|SEE_MASK_FLAG_NO_UI;
		si.lpVerb = NULL; /* Possibly should use and set to 'open' */
		si.lpFile = pszPath;
		si.lpParameters = pszParams;
		si.lpDirectory = pszDir;
		si.nShow = SW_SHOWNORMAL; /*SW_SHOWDEFAULT;*/

		ShellExecuteEx(&si);
	}
	return;
}


/*--------RunItem---------------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
HANDLE WINAPI RunItem(
	 LPCTSTR pszDir
	,LPCTSTR pszPath
)
{
	HANDLE hProcess = NULL;

	if( NULL != pszPath && 0 != *pszPath )
	{
		PROCESS_INFORMATION pi;
		STARTUPINFO si;
		_TCHAR tmpPath[MAX_PATH] = {0};
		_TCHAR szDir[MAX_PATH] = {0};

		if( NULL == pszDir && NULL != _tcschr(pszPath, _T('\\')) )
		{
			UINT uFileLen = _tcslen(pszPath);

			while( 0 != uFileLen && pszPath[uFileLen-1] != _T('\\') )
			{
				--uFileLen;
			}

			if( 0 != uFileLen && uFileLen < MAX_PATH )
			{
				pszDir = _tcsncpy(szDir, pszPath, uFileLen);
				szDir[uFileLen] = 0;
			}

			while(_T('\"') == *pszDir && 0 != *pszDir)
			{
				pszDir++;
			}
		}

		/*----------------------------------------------------------------------
		 CreateProcess modifies this param, so copy into temp buffer
		----------------------------------------------------------------------*/
		_tcscpy(tmpPath, pszPath);

		memset(&pi, 0, sizeof(PROCESS_INFORMATION));
		memset(&si, 0, sizeof(STARTUPINFO));

		si.cb = sizeof(PROCESS_INFORMATION);
		if(CreateProcess(
			NULL,
			tmpPath,
			NULL,
			NULL,
			FALSE,
			CREATE_DEFAULT_ERROR_MODE|NORMAL_PRIORITY_CLASS,
			NULL,
			pszDir,
			&si,
			&pi
		))
		{
			/*------------------------------------------------------------------
			 Clean up
			------------------------------------------------------------------*/
			CloseHandle(pi.hThread);
			//CloseHandle(pi.hProcess);
			hProcess = pi.hProcess;
		}
#ifdef _DEBUG
		else
		{
			DWORD dwErr = GetLastError();
			_stprintf(
				 szDir
				,_T("CreateProcess failed (0x%08X): %s\r\n"), dwErr, tmpPath
			);
			OutputDebugString(szDir);
		}
#endif
	}

	return(hProcess);
}


/*--------RunFolderContents-----------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID WINAPI RunFolderContents(
	 LPCTSTR pszFolder
	,LPCTSTR pszFilter
)
{
	if(
		NULL != pszFolder
		&&
		0 != *pszFolder
		&&
		(GetFileAttributes(pszFolder) & FILE_ATTRIBUTE_DIRECTORY)
	)
	{
		_TCHAR szDir[MAX_PATH], szPath[MAX_PATH], szFile[MAX_PATH];
		struct _tfinddata_t finddata;
		LONG search_handle;

		_tcscpy(szPath, pszFolder);

		if( _T('\\') != szPath[_tcslen(szPath)-1] )
		{
			_tcscat(szPath, _T("\\"));
		}

		_tcscpy(szDir, szPath);
		_tcscat(
			szPath,
			(NULL != pszFilter && 0 != *pszFilter ? pszFilter:_T("*.*"))
		);

		search_handle = _tfindfirst(szPath, &finddata);
		while( -1 != search_handle )
		{
			if(
				_tcsicmp(finddata.name, _T("."))
				&&
				_tcsicmp(finddata.name, _T(".."))
			)
			{
				_tcscpy(szFile, szDir);
				_tcscat(szFile, finddata.name);

				/*--------------------------------------------------------------
				 We are assuming the File is a valid file so are not looking at
				 for a 0xFFFFFFFF (-1) error return value
				--------------------------------------------------------------*/
				if(
					0 == (
						GetFileAttributes(szFile)
						&
						(
							FILE_ATTRIBUTE_DIRECTORY
							|
							FILE_ATTRIBUTE_HIDDEN
							|
							FILE_ATTRIBUTE_SYSTEM
						)
					)
				)
				{
					RunShellItem(szDir, szFile, NULL);
				}
			}

			if( -1 == _tfindnext(search_handle, &finddata) )
			{
				_findclose(search_handle);
				search_handle = -1;
			}
		}
	}

	return;
}


/*--------RunStartupMenu--------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID WINAPI RunStartupMenu(
	VOID
)
{
	const UINT STARTUPMENU_TABLE[] =
    {
        CSIDL_COMMON_STARTUP,
        CSIDL_COMMON_ALTSTARTUP,
        CSIDL_STARTUP,
        CSIDL_ALTSTARTUP
    };
	const ULONG STARTUPMENU_SIZE = sizeof(STARTUPMENU_TABLE) / sizeof(UINT);

	LPMALLOC pMalloc = NULL;

	if(NOERROR == SHGetMalloc(&pMalloc))
	{
		_TCHAR szPath[MAX_PATH];
		LPITEMIDLIST piil;
		HRESULT hResult;
		DWORD dwIndex;

		for(dwIndex = 0; dwIndex < STARTUPMENU_SIZE; dwIndex++)
		{
			hResult = SHGetSpecialFolderLocation(
				 NULL
				,STARTUPMENU_TABLE[dwIndex]
				,&piil
			);

			if(S_OK == hResult)
			{
				if(FALSE != SHGetPathFromIDList(piil, szPath))
				{
					RunFolderContents(szPath, NULL);
				}

				pMalloc->lpVtbl->Free(pMalloc, piil);
			}
		}
	}

	if(NULL != pMalloc)
	{
		pMalloc->lpVtbl->Release(pMalloc);
	}

	return;
}


/*--------RunRegKeys------------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID WINAPI RunRegKeys(
	 HKEY hKey
	,LPCTSTR pszSubKey
	,BOOL bDelete
	,BOOL bWait
)
{
	LONG lResult;
	HKEY hSubKey = NULL;

	lResult = RegOpenKeyEx(
		 hKey
		,pszSubKey
		,0
		,MAXIMUM_ALLOWED
		,&hSubKey
	);

	if(ERROR_SUCCESS == lResult)
	{
		HANDLE hProcess;

		DWORD dwIndex;
		DWORD dwType;

		_TCHAR szValueName[256];
		_TCHAR szValueData[MAX_PATH];

		DWORD dwValueName;
		DWORD dwValueData;

		dwIndex = 0;

		do
		{
			dwValueName = 256;
			dwValueData = MAX_PATH;

			lResult = RegEnumValue(
				 hSubKey
				,dwIndex++
				,szValueName
				,&dwValueName
				,NULL
				,&dwType
				,(LPBYTE)szValueData
				,&dwValueData
			);

			if(
				ERROR_SUCCESS == lResult
				&&
				(REG_SZ == dwType || REG_EXPAND_SZ == dwType)
			)
			{
				if(bDelete && _T('!') != *szValueName)
				{
					if(ERROR_SUCCESS == RegDeleteValue(hSubKey, szValueName))
					{
						dwIndex--;
					}
				}
	
				hProcess = RunItem(NULL, szValueData);

				if(bWait && NULL != hProcess)
				{
					WaitForSingleObject(hProcess, INFINITE);
				}

				if(NULL != hProcess)
				{
					CloseHandle(hProcess);

					if(bDelete && _T('!') == *szValueName)
					{
						if(ERROR_SUCCESS == RegDeleteValue(hSubKey,szValueName))
						{
							dwIndex--;
						}
					}
				}
			}
		}while(ERROR_NO_MORE_ITEMS != lResult);

		RegCloseKey(hSubKey);
	}

	return;
}

/*--------_runOnceEx------------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
static VOID _runOnceEx(
	VOID
)
{
	_TCHAR szBuffer[MAX_PATH+32];
	UINT unBuffLen;

	unBuffLen = GetSystemDirectory(szBuffer, MAX_PATH);

	if(0 != unBuffLen && MAX_PATH > unBuffLen)
	{
		HMODULE hROnceLib;

		_tcscat(szBuffer, _T("\\iernonce.dll"));

		hROnceLib = LoadLibraryEx(szBuffer, NULL, LOAD_LIBRARY_AS_DATAFILE);

		if(NULL != hROnceLib)
		{
			FreeLibrary(hROnceLib);

			_tcscat(szBuffer, _T(",RunOnceExProcess"));

			ShellExecute(
				 NULL
				,_T("open")
				,_T("rundll32.exe")
				,szBuffer
				,NULL
				,SW_SHOWNORMAL
			);
		}
	}

	return;
}


/*------------------------------------------------------------------------------
 Startup Order according to MSDN
  - http://support.microsoft.com/support/kb/articles/q179/3/65.asp
--------------------------------------------------------------------------------
 All keys are supposed to load asynchronously, except for

  HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunOnce

 which is run synchronously.

 The system loads the RunServices(Once) keys (under 9x)
--------------------------------------------------------------------------------

HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunServicesOnce 

HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunServices 

<Logon Prompt> 

<Shell loads, startup/initialization routines>
<Shell begins running startup items>

HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunOnce 

<Shell loads user interface>

HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Run 

HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Run 

StartUp Folders  ( I guess All users first, then Current User )

HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\RunOnce 

------------------------------------------------------------------------------*/

VOID RunStartup(
	HANDLE hStartupEvent
)
{
	MS_SHRestricted fpSHRestricted;
	HANDLE hShellReadyEvent;

	//hShellReadyEvent = CreateEvent(
	//	 NULL
	//	,TRUE
	//	,FALSE
	//	,_T("ShellIsReadyEvent")
	//);
	//if(ERROR_ALREADY_EXISTS != GetLastError())
	//{
	//	CloseHandle(hShellReadyEvent);
	//	hShellReadyEvent = NULL;
	//}
	hShellReadyEvent = OpenEvent(
		 SYNCHRONIZE
		,FALSE
		,_T("ShellIsReadyEvent")
	);

	fpSHRestricted = (MS_SHRestricted)GetProcAddress(
		 GetModuleHandle(_T("SHELL32"))
		,MAKEINTRESOURCEA(0x0064)
	);

	/*--------------------------------------------------------------------------
	 Run HKEY_LOCAL_MACHINE 'RunOnce' applications, and delete from registry
	 [eg, driver/system setup stuff]
	--------------------------------------------------------------------------*/
#ifdef USE_STARTUP_POLICIES
	if(
		NULL == fpSHRestricted
		||
		0 == fpSHRestricted(REST_NOLOCALMACHINERUNONCE)
	)
#endif
	{
		RunRegKeys(HKEY_LOCAL_MACHINE, REGSTR_PATH_RUNONCE, TRUE, TRUE);
	}

	_runOnceEx();

	/*--------------------------------------------------------------------------
	 Set an event saying HKLM 'RunOnce' has finished
	--------------------------------------------------------------------------*/
	if(NULL != hStartupEvent)
	{
		SetEvent(hStartupEvent);
	}

	/*--------------------------------------------------------------------------
	 Wait for the shell to finish loading
	--------------------------------------------------------------------------*/
	if(NULL != hShellReadyEvent)
	{
		WaitForSingleObject(hShellReadyEvent, INFINITE);
	}

	/*--------------------------------------------------------------------------
	 Run HKEY_LOCAL_MACHINE 'Run' items. Do not delete
	--------------------------------------------------------------------------*/
#ifdef USE_STARTUP_POLICIES
	if(
		NULL == fpSHRestricted
		||
		0 == fpSHRestricted(REST_NOLOCALMACHINERUN)
	)
#endif
	{
		RunRegKeys(HKEY_LOCAL_MACHINE, REGSTR_PATH_RUN, FALSE, FALSE);
	}

	RunRegKeys(HKEY_LOCAL_MACHINE, REGSTR_PATH_POLICIES_RUN, FALSE, FALSE);

	/*--------------------------------------------------------------------------
	 Run HKEY_CURRENT_USER 'Run' items. Do not delete
	--------------------------------------------------------------------------*/
#ifdef USE_STARTUP_POLICIES
	if(
		NULL == fpSHRestricted
		||
		0 == fpSHRestricted(REST_NOCURRENTUSERRUN)
	)
#endif
	{
		RunRegKeys(HKEY_CURRENT_USER, REGSTR_PATH_RUN, FALSE, FALSE);
	}

	RunRegKeys(HKEY_CURRENT_USER, REGSTR_PATH_POLICIES_RUN, FALSE, FALSE);

	/*--------------------------------------------------------------------------
	 Run items in the Startup folder(s)
	--------------------------------------------------------------------------*/
	RunStartupMenu();

	/*--------------------------------------------------------------------------
	 Run HKEY_CURRENT_USER 'RunOnce' applications, and delete from registry
	 [eg, user application setup stuff]
	--------------------------------------------------------------------------*/
#ifdef USE_STARTUP_POLICIES
	if(
		NULL == fpSHRestricted
		||
		0 == fpSHRestricted(REST_NOCURRENTUSERRUNONCE)
	)
#endif
	{
		RunRegKeys(HKEY_CURRENT_USER, REGSTR_PATH_RUNONCE, TRUE, FALSE);
	}

	return;
}

/*--------SS_RunStartup---------------------------------------------------------
 Purpose:
 - To run all startup items as specified by MS
 - HANDLE hEvent is used for notifying the caller when the
   HKEY_LOCAL_MACHINE - RunOnce items have been ran.

 NOTES:
 - One should use _beginthreadex() when calling this function
------------------------------------------------------------------------------*/
UINT WINAPI SS_RunStartup(
	HANDLE hStartupEvent
)
{
	/* If startup item were previously run, do not do so again. */
	if(StartupHasBeenRun())
	{
		/*----------------------------------------------------------------------
		 Must set event, even though we aren't running startup items.
		----------------------------------------------------------------------*/
		if(NULL != hStartupEvent)
		{
			SetEvent(hStartupEvent);
		}
	}
	else
	{
		RunStartup(hStartupEvent);
	}

	_endthreadex(0);
	return(0);
}

/*--------SS_ForceStartup-------------------------------------------------------
 Purpose:
 - To run all startup items as specified by MS
 - HANDLE hEvent is used for notifying the caller when the
   HKEY_LOCAL_MACHINE - RunOnce items have been ran.

 NOTES:
 - One should use _beginthreadex() when calling this function
------------------------------------------------------------------------------*/
UINT WINAPI SS_ForceStartup(
	HANDLE hStartupEvent
)
{
	/* Call StartupHasBeenRun, so that the flag is set. */
	StartupHasBeenRun();
	/* Run Startup items regardless if StartupHasBeenRun is TRUE or not. */
	RunStartup(hStartupEvent);

	_endthreadex(0);
	return(0);
}

/*--------End Of File---------------------------------------------------------*/
